# busco 4 output dir information parser

import os
from lib.xuyuxing.base.common_command import merge_dict
from lib.common.fileIO import tsv_file_dict_parse
from lib.xuyuxing.seq.genome_feature import ChrLoci, section_of_chr_loci
from api.xuyuxing.genome.hmmer import hmmsearch_domtblout_parse
from BCBio import GFF


def read_full_table(full_table_file, mode):
    """
    full_table_file = '/lustre/home/xuyuxing/Database/Gel/genome/annotation/maker/round1/Gel.frag.1.genome_vs_embryophyta_odb10/run_embryophyta_odb10/full_table.tsv'
    full_table_file = '/lustre/home/xuyuxing/Database/Gel/genome/annotation/maker_v2/round1/Gel.frag.1.maker.output/Gel_aa_vs_embryophyta_odb10/run_embryophyta_odb10/full_table.tsv'
    """
    busco_dict = {}

    if mode == 'genome':
        data_info = tsv_file_dict_parse(full_table_file,
                                        fieldnames=["Busco_id", "Status", "Sequence", "Gene_Start", "Gene_End", "Score",
                                                    "Length", "OrthoDB_url", "Description"], ignore_prefix=r'^#')

        for i in data_info:
            info = data_info[i]
            if info["Busco_id"] not in busco_dict:
                busco_dict[info["Busco_id"]] = BuscoObject(info["Busco_id"], info["Status"], [], info["OrthoDB_url"],
                                                           info["Description"])

            tmp_chr_loci = ChrLoci(
                chr_id=info["Sequence"], strand=None, start=info["Gene_Start"], end=info["Gene_End"])
            busco_dict[info["Busco_id"]].hits.append(tmp_chr_loci)

    elif mode == 'proteins':
        data_info = tsv_file_dict_parse(full_table_file,
                                        fieldnames=["Busco_id", "Status", "Sequence", "Score", "Length", "OrthoDB_url",
                                                    "Description"], ignore_prefix=r'^#')

        for i in data_info:
            info = data_info[i]
            if info["Busco_id"] not in busco_dict:
                busco_dict[info["Busco_id"]] = BuscoObject(info["Busco_id"], info["Status"], [], info["OrthoDB_url"],
                                                           info["Description"])

            busco_dict[info["Busco_id"]].hits.append(info["Sequence"])

    return busco_dict


def load_augustus_output(augustus_output_dir):
    """
    augustus_output_dir = "/lustre/home/xuyuxing/Database/Gel/genome/annotation/maker/round1/Gel.frag.1.genome_vs_embryophyta_odb10/run_embryophyta_odb10/augustus_output"
    """

    # gff
    gff_dir = augustus_output_dir + "/gff"

    augustus_range_dict = {}
    for gff_file in os.listdir(gff_dir):
        if gff_file.split('.')[-1] == 'gff':
            query_busco_id = gff_file.split('.')[0]
            gff_file = gff_dir + "/" + gff_file
            chr_loci_list = []
            with open(gff_file, 'r') as in_handle:
                for rec in GFF.parse(in_handle):
                    for gene in rec.features:
                        if gene.type != 'gene':
                            continue
                        start = gene.location.start.position + 1
                        end = gene.location.end.position
                        chr_loci_list.append(
                            ChrLoci(chr_id=rec.id, strand=gene.strand, start=start, end=end))

            augustus_range_dict[query_busco_id] = chr_loci_list

    return augustus_range_dict


def load_blast_output(blast_output_dir):
    """
    blast_output_dir = "/lustre/home/xuyuxing/Database/Gel/genome/annotation/maker/round1/Gel.frag.1.genome_vs_embryophyta_odb10/run_embryophyta_odb10/blast_output"
    """

    coordinates_file1 = blast_output_dir + "/coordinates.tsv"
    coordinates_file2 = blast_output_dir + "/coordinates_missing_and_frag_rerun.tsv"

    info_dict1 = tsv_file_dict_parse(coordinates_file1, fieldnames=[
                                     'busco_id', 'contig', 'start', 'end'], prefix="A_")
    info_dict2 = tsv_file_dict_parse(coordinates_file2, fieldnames=[
                                     'busco_id', 'contig', 'start', 'end'], prefix="B_")

    info_dict = merge_dict([info_dict1, info_dict2], False)

    blast_range_dict = {}
    for id in info_dict:
        info = info_dict[id]
        if info['busco_id'] not in blast_range_dict:
            blast_range_dict[info['busco_id']] = []

        chr_loci_tmp = ChrLoci(
            chr_id=info['contig'], strand=None, start=info['start'], end=info['end'])

        overlap_loci = []
        for i in blast_range_dict[info['busco_id']]:
            if section_of_chr_loci(chr_loci_tmp, i)[0]:
                overlap_loci.append(i)

        for i in overlap_loci:
            blast_range_dict[info['busco_id']].remove(i)

        overlap_loci.append(chr_loci_tmp)

        blast_range_dict[info['busco_id']].append(
            sorted(overlap_loci, key=lambda x: x.length, reverse=True)[0])

    return blast_range_dict


def load_hmmer_output(hmmer_output_dir):
    """
    hmmer_output_dir = "/lustre/home/xuyuxing/Database/Gel/genome/annotation/maker/round1/Gel.frag.1.genome_vs_embryophyta_odb10/run_embryophyta_odb10/hmmer_output"
    """

    hmmer_range_dict = {}
    for hmmer_file in os.listdir(hmmer_output_dir):
        hmmer_file = hmmer_output_dir + "/" + hmmer_file
        hmmer_info = hmmsearch_domtblout_parse(hmmer_file)

        for hmmer_target_id in hmmer_info:
            hmmer_target = hmmer_info[hmmer_target_id]
            for domin in hmmer_target.HmmDomain_list:
                q_name = domin.q_name
                if q_name not in hmmer_range_dict:
                    hmmer_range_dict[q_name] = []
                hmmer_range_dict[q_name].append(hmmer_target_id)

    for i in hmmer_range_dict:
        hmmer_range_dict[i] = list(set(hmmer_range_dict[i]))

    return hmmer_range_dict


class BuscoObject(object):
    def __init__(self, id, status=None, hits=None, OrthoDB_url=None, description=None, evidence=None):
        self.id = id
        self.status = status
        self.hits = hits
        self.OrthoDB_url = OrthoDB_url
        self.description = description
        self.evidence = evidence


class BuscoResults(object):
    def __init__(self, busco_output_dir, mode, lineage, busco_datadir=None, modify_dict={}):
        # input
        self.dir = busco_output_dir
        self.mode = mode
        self.lineage = lineage
        self.busco_datadir = busco_datadir

        # get cwd and relpath of dir
        self.abspath = os.path.abspath(self.dir)
        self.cwd = os.getcwd()

        # get important dir and file in busco output
        self.busco_run_dir = self.abspath + "/run_" + self.lineage

        self.full_table = self.busco_run_dir + "/full_table.tsv"

        self.evidence_dir = {}
        self.evidence_dir['hmmer'] = self.busco_run_dir + "/hmmer_output"

        # only in genome mode
        if self.mode == 'genome':
            self.evidence_dir['augustus'] = self.busco_run_dir + \
                "/augustus_output"
            self.evidence_dir['blast'] = self.busco_run_dir + "/blast_output"

        # from modify_dict to change defaults attr
        for new_attr in modify_dict:
            if hasattr(self, new_attr):
                setattr(self, new_attr, modify_dict[new_attr])

        # if have busco_datadir
        if self.busco_datadir:
            self.lineage_datadir = self.busco_datadir + "/lineages/" + self.lineage

        # load results
        self.load_busco_results()

    def load_busco_results(self):
        # load full table
        self.busco_dict = read_full_table(self.full_table, self.mode)

        # load evidence_dir
        # for genome mode
        if self.mode == 'genome':
            # augustus output
            augustus_range_dict = load_augustus_output(
                self.evidence_dir['augustus'])

            # blast output
            blast_range_dict = load_blast_output(self.evidence_dir['blast'])

            # hmmer output
            hmmer_range_dict = load_hmmer_output(self.evidence_dir['hmmer'])

            for busco_id in self.busco_dict:
                self.busco_dict[busco_id].evidence = {}

                if busco_id in augustus_range_dict:
                    self.busco_dict[busco_id].evidence['augustus'] = augustus_range_dict[busco_id]
                else:
                    self.busco_dict[busco_id].evidence['augustus'] = None

                if busco_id in blast_range_dict:
                    self.busco_dict[busco_id].evidence['blast'] = blast_range_dict[busco_id]
                else:
                    self.busco_dict[busco_id].evidence['blast'] = None

                if busco_id in hmmer_range_dict:
                    self.busco_dict[busco_id].evidence['hmmer'] = hmmer_range_dict[busco_id]
                else:
                    self.busco_dict[busco_id].evidence['hmmer'] = None

        # for protein mode
        elif self.mode == 'proteins':
            # hmmer output
            hmmer_range_dict = load_hmmer_output(self.evidence_dir['hmmer'])

            for busco_id in self.busco_dict:
                self.busco_dict[busco_id].evidence = {}

                if busco_id in hmmer_range_dict:
                    self.busco_dict[busco_id].evidence['hmmer'] = hmmer_range_dict[busco_id]
                else:
                    self.busco_dict[busco_id].evidence['hmmer'] = None


if __name__ == '__main__':
    # parse orthofinder dir
    busco_output_dir = '/lustre/home/xuyuxing/Database/Gel/genome/annotation/maker/round1/Gel.frag.1.genome_vs_embryophyta_odb10'
    mode = 'genome'
    lineage = 'embryophyta_odb10'
    busco_datadir = '/lustre/home/xuyuxing/Database/Gel/genome/annotation/maker/round1/busco_downloads'

    # load data dir
    busco_results = BuscoResults(
        busco_output_dir, mode, lineage, busco_datadir=busco_datadir, modify_dict={})

    # count status number
    from collections import Counter
    busco_status = Counter(
        [busco_results.busco_dict[i].status for i in busco_results.busco_dict])

    complete_buscos = [
        i for i in busco_results.busco_dict if busco_results.busco_dict[i].status == 'Complete']
    duplicated_buscos = [
        i for i in busco_results.busco_dict if busco_results.busco_dict[i].status == 'Duplicated']
    fragmented_buscos = [
        i for i in busco_results.busco_dict if busco_results.busco_dict[i].status == 'Fragmented']
    missing_buscos = [
        i for i in busco_results.busco_dict if busco_results.busco_dict[i].status == 'Missing']

    # parse orthofinder dir
    busco_output_dir = '/lustre/home/xuyuxing/Work/Orobanchaceae/phylogenetic/busco/Aca_vs_embryophyta_odb10'
    mode = 'proteins'
    lineage = 'embryophyta_odb10'
    busco_datadir = '/lustre/home/xuyuxing/Work/Orobanchaceae/phylogenetic/busco/busco_downloads'

    # load data dir
    busco_results = BuscoResults(
        busco_output_dir, mode, lineage, busco_datadir=busco_datadir, modify_dict={})

    # count status number
    from collections import Counter
    busco_status = Counter(
        [busco_results.busco_dict[i].status for i in busco_results.busco_dict])

    complete_buscos = [
        i for i in busco_results.busco_dict if busco_results.busco_dict[i].status == 'Complete']
    duplicated_buscos = [
        i for i in busco_results.busco_dict if busco_results.busco_dict[i].status == 'Duplicated']
    fragmented_buscos = [
        i for i in busco_results.busco_dict if busco_results.busco_dict[i].status == 'Fragmented']
    missing_buscos = [
        i for i in busco_results.busco_dict if busco_results.busco_dict[i].status == 'Missing']



data = {'a': np.arange(50),
        'c': np.random.randint(0, 50, 50),
        'd': np.random.randn(50)}

data['b'] = data['a'] + 10 * np.random.randn(50)
data['d'] = np.abs(data['d']) * 100

plt.scatter('a', 'b', c='c', s='d', data=data)

# 这里data读入了一个字典数据，这个时候其它参数就可以通过keys来访问这些数据

plt.xlabel('entry a')
plt.ylabel('entry b')
plt.show()



if __name__ == '__main__':
    import argparse
    import textwrap

    # argument parse
    # ---------------------------------------------------------
    parser = argparse.ArgumentParser(
        prog='GenCompare',
    )

    subparsers = parser.add_subparsers(
        title='subcommands', dest="subcommand_name")

    # argparse for BuildScaffoldDir
    parser_a = subparsers.add_parser('BuildScaffoldDir',
                                     help='build a dir to store scaffold orthogroups for other analysis\n',
                                     description='')

    parser_a.add_argument('orthogroups_file', type=str,
                          help='path of orthogroups tsv file')
    parser_a.add_argument('species_info_file', type=str,
                          help='species_info_file need have sp_id, cds_file and pt_file')
    parser_a.add_argument('scaffold_dir', type=str, help='output scaffold dir')
    parser_a.add_argument("-t", "--num_threads",
                          help="thread number (default as 1)", default=1, type=int)
    parser_a.add_argument("-g", "--min_gene_number",
                          help="min_gene_number (default as 4)", default=4, type=int)
    parser_a.add_argument("-s", "--min_species_number",
                          help="min_species_number (default as 2)", default=2, type=int)

    # argparse for SeqClassify
    parser_a = subparsers.add_parser('SeqClassify',
                                     help='map genome proteins to builded gene family\n',
                                     description='')

    parser_a.add_argument('input_fasta', type=str,
                          help='input genome.protein.fasta file')
    parser_a.add_argument('scaffold_dir', type=str,
                          help='dir from BuildScaffoldDir')
    parser_a.add_argument('output_file', type=str, help='output dir')
    parser_a.add_argument("-w", "--work_dir",
                          help="work dir for tmp (default as tmp)", default=None, type=str)
    parser_a.add_argument("-t", "--thread_number",
                          help="thread number (default as 1)", default=1, type=int)
    parser_a.add_argument("-s", "--strict",
                          help="strict flag: will use msa and pvalue to check False positive", action='store_true')


    # argparse for TransDeepAssembly
    parser_a = subparsers.add_parser('TransDeepAssembly',
                                     help='map transcriptome assemblies to builded orthogroup, and re-assemble, from AssemblyPostProcessor in PlantTribes\n',
                                     description='')

    parser_a.add_argument('input_fasta', type=str,
                          help='input Trinity.fasta file')
    parser_a.add_argument('output_dir', type=str, help='output dir')
    parser_a.add_argument("-t", "--thread_number",
                          help="thread number (default as 1)", default=1, type=int)

    # argparse for FamMapStats
    parser_a = subparsers.add_parser('FamMapStats',
                                     help='check how many orthogroup can get good hit\n',
                                     description='')

    parser_a.add_argument('family_dir', type=str,
                          help='family dir from TransDeepAssembly or MapGenomeToOrthoGroups')
    parser_a.add_argument("-t", "--thread_number",
                          help="thread number (default as 56)", default=56, type=int)

    # argparse for GetSkipFile
    parser_a = subparsers.add_parser('GetSkipFile',
                                     help='get skip file for WPGmapper')
    parser_a.add_argument(
        "gff_file", help="gene gff file", type=str)
    parser_a.add_argument(
        "repeatmasker_out", help="repeatmasker output file", type=str)
    parser_a.add_argument(
        "output_file", help="output skip file", type=str)

    # argparse for WPGmapper
    parser_a = subparsers.add_parser('WPGmapper',
                                     help='Wide Proteins To Genome Maper (genblasta and genewise)')
    parser_a.add_argument("reference_genome_table",
                          help="Path of reference genome table in excel format, must have column \"id\" and \"pt_file\"",
                          type=str)
    parser_a.add_argument("target_genome_fasta",
                          help="Path of target genome fasta file", type=str)
    parser_a.add_argument("-skip", "--skip_range_file",
                          help="Path of skip_range_file, tsv file chr start end", type=str)
    parser_a.add_argument("-d", "--work_dir", help="Path of work dir (default as ./WPG_working)",
                          default="./WPG_working", type=str)
    parser_a.add_argument(
        "-t", "--num_threads", help="threads number (default as 5)", default=5, type=int)
    parser_a.add_argument("-c", "--gene_coverage",
                          help="gene coverage (default as 0.2)", default=0.2, type=float)
    parser_a.add_argument("-n", "--genblasta_hit_num",
                          help="genblasta hit num (default as 100)", default=100, type=int)
    parser_a.add_argument("-a", "--annotated_coverage", help="annotated_coverage (default as 0.8)", default=0.8,
                          type=float)
    parser_a.add_argument("-s", "--split_seq_num", help="split big fasta file to run (default as 1000)", default=1000,
                          type=int)
    parser_a.add_argument("-r", "--force_redo",
                          help="force redo all job", action='store_true')

    # argparse for GeneLoss
    parser_a = subparsers.add_parser('GeneLoss',
                                     help='Parse gene loss from a orthofinder results, with tree check')

    parser_a.add_argument(
        "config_file", help="Path of configure file", type=str)
    parser_a.add_argument(
        "--ortho_level", help="ortho_level: orthogroups or orthologues (default:orthologues)")
    parser_a.add_argument("--num_threads", help="number of threads", type=int)
    parser_a.add_argument("--log_file", help="path of log file", type=str)
    parser_a.add_argument("--output_dir", help="path of output dir", type=str)

    # argparse for GeneLoss2
    parser_a = subparsers.add_parser('GeneLoss2',
                                     help='Parse gene loss from a given orthogroups file, check by scaffold dir')

    parser_a.add_argument(
        "--config_file", help="Path of configure file", type=str)
    parser_a.add_argument("--orthogroup_scaffold_dir",
                          help="orthogroup_scaffold_dir from BuildScaffoldDir", type=str)
    parser_a.add_argument(
        "--wpgmapper_dir", help="wpgmapper_dir from WPGmapper", type=str)
    parser_a.add_argument(
        "--target_speci", help="species id for target species", type=str)
    parser_a.add_argument("--target_speci_annotation",
                          help="target species gff file", type=str)
    parser_a.add_argument(
        "--conserved_species", help="set species need for defend conserved orthogroups, like: Ath;Ini;Llu;Cca,Sly,Oeu,Sin,Mgu,3", type=str)
    parser_a.add_argument("--work_dir", help="output dir", type=str)
    parser_a.add_argument("--log_file", help="log file", type=str)

    parser_a.add_argument(
        "--num_threads", help="number of threads default=56", type=int, default=56)
    parser_a.add_argument(
        "--top_evidence_num", help="use top_evidence_num evidences for a ref gene default=20", type=int, default=20)
    parser_a.add_argument(
        "--min_cover", help="min coverage default=0.5", type=float, default=0.5)
    parser_a.add_argument(
        "--min_aa_len", help="min protein length default=20", type=int, default=20)
    parser_a.add_argument(
        "--min_identity", help="min_identity default=0.3", type=float, default=0.3)
    parser_a.add_argument("--min_score", help="min_score default=50.0",
                          type=float, default=50.0)
    parser_a.add_argument("--target_speci_feature",
                          help="target_speci_feature in gff file default=gene", type=str, default='gene')
    parser_a.add_argument("--annotated_coverage",
                          help="annotated_coverage default=0.8", type=float, default=0.8)

    # argparse for GeneFamilyStat
    parser_a = subparsers.add_parser('GeneFamilyStat',
                                     help='parse gene family count file to get gene family expansion and contraction')

    parser_a.add_argument('orthogroups_genecount_tsv', type=str,
                          help='path of Orthogroups_GeneCount_tsv from orthofinder')
    parser_a.add_argument('species_tree', type=str,
                          help='a species tree file in newick')
    parser_a.add_argument('species_name_list', type=str,
                          help='a species name list, seq by \",\"')
    parser_a.add_argument('conserved_species_name_list', type=str,
                          help='a conserved species name list, seq by \",\"')
    parser_a.add_argument('work_dir', type=str, help='temp working dir')
    parser_a.add_argument('output_prefix', type=str, help='output_prefix')
    parser_a.add_argument("-t", "--threads", default=56, type=int)
    parser_a.add_argument('-l', '--just_load',
                          help='just_load', action='store_true')

    # argparse for EasyBadiRate
    parser_a = subparsers.add_parser('EasyBadiRate',
                                     help='Parse one gene family expansion or contraction')

    parser_a.add_argument('tag', type=str, help='a tag for this family')
    parser_a.add_argument('tree_file', type=str,
                          help='a species tree file in newick')
    parser_a.add_argument('size_tsv_file', type=str,
                          help='a gene family size file in tsv file')
    parser_a.add_argument('-l', '--label_tree', type=str,
                          help='output a labeled tree from BadiRate', default=None)
    parser_a.add_argument('-k', '--keep_tmp_dir',
                          help='keep temp running dir', action='store_true')

    # argparse for hWGDdetector
    parser_a = subparsers.add_parser('hWGDdetector',
                                     help='Detect hidden WGD in many species')

    parser_a.add_argument('sp_info_excel', type=str, help='must have sp_id (same as OG_tsv_file), pt_file, cds_file (for treebset), gff_file, genome_file (for dotplot)')
    parser_a.add_argument('sp_tree_file', type=str,
                          help='a species tree file in newick, each node should have name, branch length can be ignored')
    parser_a.add_argument('OG_tsv_file', type=str,
                          help='Orthogroups file from orthofinder')
    parser_a.add_argument('-w', '--work_dir', type=str, help='work dir', default='hWGDdetector_dir')
    parser_a.add_argument('-n', '--huge_OG_gene_num', type=int,
                          help='orthogroup with single species have gene more than h will be ignored', default=20)
    parser_a.add_argument('-t', '--threads',
                          help='number of threads', type=int, default=56)

    args = parser.parse_args()
    args_dict = vars(args)

    # ---------------------------------------------------------
    # command detail

    # BuildScaffoldDir
    if args_dict["subcommand_name"] == "BuildScaffoldDir":
        from api.xuyuxing.comp_genome.planttribes2 import build_scaffold_main
        build_scaffold_main(args)

    # SeqClassify
    elif args_dict["subcommand_name"] == "SeqClassify":
        from api.xuyuxing.comp_genome.planttribes2 import seq_classify_main
        seq_classify_main(args)

    # TransDeepAssembly
    elif args_dict["subcommand_name"] == "TransDeepAssembly":
        from api.xuyuxing.comp_genome.planttribes import targeted_gene_family_assembly
        targeted_gene_family_assembly(args)

    # GetDeepAssemblyStats
    elif args_dict["subcommand_name"] == "GetDeepAssemblyStats":
        from api.xuyuxing.comp_genome.planttribes import GetDeepAssemblyStats_main
        GetDeepAssemblyStats_main(args)

    # EasyBadiRate
    elif args_dict["subcommand_name"] == "EasyBadiRate":
        import os
        import uuid
        from config import badirate_path
        from api.xuyuxing.evolution.badirate import main_pipeline

        main_pipeline(args.tag, args.size_tsv_file, args.tree_file, "/tmp/" +
                      uuid.uuid1().hex, badirate_path, args.label_tree, args.keep_tmp_dir)

    # GeneLoss
    elif args_dict["subcommand_name"] == "GeneLoss":
        from src.xuyuxing.GenomeTools.GeneLoss import GeneLoss_args_parser
        GeneLoss_args_parser(args)

    # GeneLoss2
    elif args_dict["subcommand_name"] == "GeneLoss2":
        from src.xuyuxing.GenomeTools.GeneLoss2 import GeneLoss_args_parser
        GeneLoss_args_parser(args)

    # GeneFamilyStat
    elif args_dict["subcommand_name"] == "GeneFamilyStat":
        from src.xuyuxing.GenomeTools.GeneFamily import gene_family_stat_main

        gene_family_stat_main(args.orthogroups_genecount_tsv, args.species_tree, args.species_name_list.split(
            ","), args.conserved_species_name_list.split(","), args.work_dir, args.threads, args.work_dir+"/log", args.output_prefix, args.just_load)

    # WideProteinToGenomeMaper
    elif args_dict["subcommand_name"] == "WPGmapper":
        from src.xuyuxing.GenomeTools.WPGmapper import build_WPGmapper
        build_WPGmapper(args)

    # GetSkipFile
    elif args_dict["subcommand_name"] == "GetSkipFile":
        from src.xuyuxing.GenomeTools.WPGmapper import get_skip_file
        get_skip_file(args.gff_file, args.repeatmasker_out, args.output_file)

    # hWGDdetector
    elif args_dict["subcommand_name"] == "hWGDdetector":
        from src.xuyuxing.GenomeTools.hWGDdetector import hWGDdetector_main
        args.log_file = args.work_dir + "/log"
        hWGDdetector_main(args)



"""
The script hopes to get the genome information from JGI Genome Portal

All JGI data were recorded in a file named genome_project.csv

genome-projects.csv can be download from JGI Genome Portal
login https://genome.jgi.doe.gov/portal
Advanced Search --> show all --> Report --> Download Project Overview Report

But genome-projects.csv have many other information which not about genome,
so records in genome-project.csv should be filtered. This script filter the records
by Taxonomy ID in the record, e.g. If record have Taxonomy ID and this ID is accurate than a
specific rank in NCBI Taxonomy, such as accurate than "genus" or "species", then record would be saved

Usage:
from ncbi_taxonomy import build_taxon_database
tax_record_dict = build_taxon_database(path_for_taxonomy)

from JGI import parser_of_genome_projects
species_list = parser_of_genome_projects(genome_projects_csv_file, tax_record_dict, rank="genus")


Yuxing Xu
2018.05.08  new script
"""

__author__ = 'Yuxing Xu'

import re
import xml.etree.ElementTree as ET

import lib.common.os
import api.xuyuxing.file_parser.fileIO as fr
import lib.xuyuxing.base.base_function as bf
import os


def Portal_ID_get(Portal_ID_raw):
    return re.findall("\"(.*)\"", Portal_ID_raw.split(",")[1])[0]


def Taxonomy_ID_get(genome_projects_record):
    """
    some of record in JGI genome_project.csv file have very long taxonomy id,
    because they are repeated by many Sequencing Project ID
    """
    seq_project_count = len(re.findall(r'(\S+)', genome_projects_record["Sequencing Project ID"]))
    tax_id_raw_len = len(genome_projects_record["Taxonomy ID"])

    if seq_project_count == 0:
        return genome_projects_record["Taxonomy ID"]

    if tax_id_raw_len % seq_project_count == 0:
        tax_id_len = int(tax_id_raw_len / seq_project_count)
        split_list = re.findall('.{%d}' % tax_id_len, genome_projects_record["Taxonomy ID"])
        if len(list(set(split_list))) == 1:
            return split_list[0]
        else:
            return genome_projects_record["Taxonomy ID"]
    return genome_projects_record["Taxonomy ID"]


def parser_of_genome_projects(genome_projects_file, tax_db_file, rank, group_taxon_id=1, keep_no_rank=True):
    from lib.xuyuxing.evolution.taxonomy import read_tax_record_dict_db

    genome_projects_dict = fr.tsv_file_parse(genome_projects_file, seq=",")
    genome_projects_dict, col_name = fr.title_tsv_to_dict(genome_projects_dict)

    project_list = [(genome_projects_dict[i]["JGI Project Id"], Portal_ID_get(genome_projects_dict[i]["Portal ID"]),
                     Taxonomy_ID_get(genome_projects_dict[i])) for i in genome_projects_dict if i != "ID_0"]

    tax_record_dict = read_tax_record_dict_db(tax_db_file, [i[2] for i in project_list])

    species_list = []
    for Project_ID, Portal_ID, Taxonomy_ID in project_list:
        if not Taxonomy_ID == "":
            if Taxonomy_ID in tax_record_dict:
                if tax_record_dict[Taxonomy_ID].is_a(group_taxon_id):
                    # print(Taxonomy_ID)
                    if tax_record_dict[Taxonomy_ID].lower_than(rank):
                        if hasattr(tax_record_dict[Taxonomy_ID], rank):
                            taxonomic_rank_tmp = getattr(tax_record_dict[Taxonomy_ID], rank)
                            species_list.append((Project_ID, Portal_ID, Taxonomy_ID, taxonomic_rank_tmp))
                        elif keep_no_rank:
                            species_list.append((Project_ID, Portal_ID, Taxonomy_ID, Taxonomy_ID))

    return species_list


"curl 'https://signon.jgi.doe.gov/signon/create' --data-urlencode 'login=USER_NAME' --data-urlencode 'password=USER_PASSWORD' -c cookies > /dev/null"


def pull_info(user_name, passwd, cookie_path, output_xml, Portal_ID, directly_download):
    cmd_string = "curl 'https://genome.jgi.doe.gov/portal/ext-api/downloads/get-directory?organism=%s' -b %s > %s" % (
        Portal_ID, cookie_path, output_xml)

    if directly_download is False:
        return cmd_string

    lib.common.os.cmd_run(cmd_string)

    if os.path.exists(output_xml) and os.path.getsize(output_xml) != 0:
        return output_xml
    else:
        login_string = "curl 'https://signon.jgi.doe.gov/signon/create' --data-urlencode 'login=%s' --data-urlencode 'password=%s' -c %s > /dev/null" % (
            user_name, passwd, cookie_path)
        lib.common.os.cmd_run(login_string)
        lib.common.os.cmd_run(cmd_string)
        return output_xml


def pull_file(user_name, passwd, cookie_path, output_file, want_url, directly_download):
    cmd_string = "curl '%s' -b %s > %s" % (
        want_url.replace("&amp;", "&"), cookie_path, output_file)

    if directly_download is False:
        return cmd_string

    lib.common.os.cmd_run(cmd_string)

    if os.path.exists(output_file) and os.path.getsize(output_file) != 0:
        return output_file
    else:
        login_string = "curl 'https://signon.jgi.doe.gov/signon/create' --data-urlencode 'login=%s' --data-urlencode 'password=%s' -c %s > /dev/null" % (
            user_name, passwd, cookie_path)
        lib.common.os.cmd_run(login_string)
        lib.common.os.cmd_run(cmd_string)
        return output_file


def xml_complete(xml_path):
    flag = 0
    if not os.path.exists(xml_path):
        return 0
    try:
        tree = ET.parse(xml_path)
        root = tree.getroot()
        flag = 1
    except:
        with open(xml_path) as f:
            strings = f.read()
            if re.match(r"User \S+ does not have access to .*", strings):
                flag = 1
    return flag


def uniq_file_record(file_list):
    def good_choose(file_list_tmp):
        for file_record in file_list_tmp:
            file_url, file_name, file_md5, tag_tmp, stat, file_type, file_size = file_record
            match_url = re.match(r'.*_JAMO.*', file_url)
            if match_url:
                return file_record
        return file_list_tmp[0]

    file_dir = {}
    for file_record in file_list:
        file_url, file_name, file_md5, tag_tmp, stat, file_type, file_size = file_record
        if not file_name in file_dir:
            file_dir[file_name] = []
        file_dir[file_name].append(file_record)

    output_list = []
    for file_name in file_dir:
        file_list = file_dir[file_name]
        output_list.append(good_choose(file_list))

    return output_list


def file_stat_check(file_stat):
    from lib.common.os import md5sum_check
    file_url, file_path, md5_string, tag_tmp, stat, file_type_tmp, file_size = file_stat
    stat = "renew"
    if md5_string == "":
        if os.path.exists(file_path) and os.path.getsize(file_path) == int(file_size):
            stat = "ok"
    if md5sum_check(file_path, md5_string):
        stat = "ok"
    file_stat_new = (file_url, file_path, md5_string, tag_tmp, stat, file_type_tmp, file_size)
    return file_stat_new


def record_num_in_gzip_fasta(file_gz):
    import gzip
    with gzip.open(file_gz, 'rt') as f:
        num = 0
        for each_line in f:
            record_head = re.match(r"^>", each_line)
            if record_head:
                num = num + 1
    return num


def best_JGI_for_group(download_jgi_fungi):
    taxon_group_dict = {}
    for Portal_ID in download_jgi_fungi:
        JGI_record_temp = download_jgi_fungi[Portal_ID]
        portal_id, taxon_id, group_id = JGI_record_temp["taxon_id"]
        for version_tmp in JGI_record_temp["annotation"]:
            pt_file = [i for i in JGI_record_temp["annotation"][version_tmp] if i[5] == "pt_file"][0]
            if not group_id in taxon_group_dict:
                taxon_group_dict[group_id] = []
            taxon_group_dict[group_id].append((pt_file, portal_id, version_tmp))

    best_JGI_for_groups_list = {}
    for group_id in taxon_group_dict:
        if len(taxon_group_dict[group_id]) > 1:
            best_one = \
                sorted(taxon_group_dict[group_id], key=lambda x: record_num_in_gzip_fasta(x[0][1]), reverse=True)[0]
        else:
            best_one = taxon_group_dict[group_id][0]
        best_JGI_for_groups_list[group_id] = (best_one[1], best_one[2])

    return best_JGI_for_groups_list


if __name__ == '__main__':
    from lib.xuyuxing.evolution.taxonomy import build_taxon_database, store_tax_record_into_sqlite

    path_for_taxonomy = "/lustre/home/xuyuxing/Database/NCBI/taxonomy"
    tax_db_file = "/lustre/home/xuyuxing/Database/NCBI/taxonomy/tax_xyx.db"
    genome_projects_csv_file = "/lustre/home/xuyuxing/Database/genome/2020/JGI/genome-projects.csv"

    # you can store taxonomy database in sqlite3 file
    tax_record_dict = build_taxon_database(path_for_taxonomy)
    store_tax_record_into_sqlite(tax_record_dict, tax_db_file)

    # 4751 for fungi
    # this list will tell you how many genome project in JGI which remove reduct in genus level in fungi
    species_list = parser_of_genome_projects(genome_projects_csv_file, tax_db_file, rank="genus", group_taxon_id="4751",
                                             keep_no_rank=False)

    genus_list = list(set([i[3] for i in species_list]))

    species_list = parser_of_genome_projects(genome_projects_csv_file, tax_db_file, rank="family",
                                             group_taxon_id="4751", keep_no_rank=False)

    family_list = list(set([i[3] for i in species_list]))

    species_list = parser_of_genome_projects(genome_projects_csv_file, tax_db_file, rank="order",
                                             group_taxon_id="4751", keep_no_rank=False)

    order_list = list(set([i[3] for i in species_list]))



if __name__ == '__main__':
    import argparse

    # argument parse
    parser = argparse.ArgumentParser(
        prog='ResourceTools',
    )

    subparsers = parser.add_subparsers(
        title='subcommands', dest="subcommand_name")

    # argparse for RNAseqFromSRA
    parser_a = subparsers.add_parser('RNAseqFromSRA',
                                     help='get RNA seq sra record id from sra database')
    parser_a.add_argument("key_word", help="key_word", type=str)
    parser_a.add_argument(
        "tmp_runinfo_file", help="I will make a tmp file, where can I save it", type=str)
    parser_a.add_argument("output_file", help="path for output_file", type=str)

    # argparse for DownloadGenomeFromNCBI
    parser_a = subparsers.add_parser('DownloadGenomeFromNCBI',
                                     help='get genome sequence from NCBI')
    parser_a.add_argument("list_of_accession_ID",
                          help="give me a list of accession id like GCA_009837245.1", type=str)
    parser_a.add_argument("ncbi_genome_dir", 
                          help="ncbi_genome_dir: dir store ncbi download sequence", type=str)
    parser_a.add_argument("-r", "--assembly_summary_refseq_file", help="assembly_summary_refseq from NCBI, download from ftp://ftp.ncbi.nlm.nih.gov/genomes/ASSEMBLY_REPORTS/assembly_summary_refseq.txt (default:in work_dir)", default=None,
                          type=str)
    parser_a.add_argument("-g", "--assembly_summary_genbank_file", help="assembly_summary_genbank from NCBI, download from ftp://ftp.ncbi.nlm.nih.gov/genomes/ASSEMBLY_REPORTS/assembly_summary_genbank.txt (default:in work_dir)", default=None,
                          type=str)
    parser_a.add_argument("-t", "--taxonomy_dir",
                          help="taxonomy dump database, download from ftp://ftp.ncbi.nih.gov/pub/taxonomy/taxdump.tar.gz (default:use value from config file", default=None, type=str)
    parser_a.add_argument("-d", "--download_type",
                          help="what file you want download (default: gff_file,pt_file,cds_file,genome_file)", default=None, type=str)
    parser_a.add_argument("-m", "--renew_md5_flag",
                          help="renew all md5 file", action='store_true')
    parser_a.add_argument("-w", "--download_way", help="what download way you want? (default:wget)",
                          default="wget", choices=['wget', 'ascp'])

    # argparse for ChooseGoodGenomeFromNCBI
    parser_a = subparsers.add_parser('ChooseGoodGenomeFromNCBI',
                                     help='Choose Good Genome From NCBI')
    parser_a.add_argument("ncbi_genome_dir", 
                          help="ncbi_genome_dir: dir store ncbi download sequence", type=str)
    parser_a.add_argument("-r", "--assembly_summary_refseq_file", help="assembly_summary_refseq from NCBI, download from ftp://ftp.ncbi.nlm.nih.gov/genomes/ASSEMBLY_REPORTS/assembly_summary_refseq.txt (default:in work_dir)", default=None,
                          type=str)
    parser_a.add_argument("-g", "--assembly_summary_genbank_file", help="assembly_summary_genbank from NCBI, download from ftp://ftp.ncbi.nlm.nih.gov/genomes/ASSEMBLY_REPORTS/assembly_summary_genbank.txt (default:in work_dir)", default=None,
                          type=str)
    parser_a.add_argument("-t", "--taxonomy_dir",
                          help="taxonomy dump database, download from ftp://ftp.ncbi.nih.gov/pub/taxonomy/taxdump.tar.gz (default:use value from config file", default=None, type=str)
    parser_a.add_argument("-tr", "--taxonomic_rank",
                          help=" Compare genomic data between taxonomic_rank classification levels", default='family', type=str)
    parser_a.add_argument("-tt", "--top_taxon",
                          help="The selected genomic source species should all belong to top_taxon", default='Magnoliopsida', type=str)
    parser_a.add_argument("-kr", "--remove_no_rank",
                          help="For those species without taxonomic_rank information in the taxonomic information will deleted (default:False)", action='store_true')
    parser_a.add_argument("-na", "--allow_no_annotation",
                          help="Genomes without annotation information will be allowed to be retained (default:False)", action='store_true')
    parser_a.add_argument("-m", "--renew_md5_flag",
                          help="renew all md5 file", action='store_true')
    parser_a.add_argument("-tn", "--top_num",
                          help="keep top_num genome assemblies in each taxonomic_rank (default:2)", default=2, type=int)
    parser_a.add_argument("-w", "--download_way", help="what download way you want? (default:wget)",
                          default="wget", choices=['wget', 'ascp'])


    args = parser.parse_args()
    args_dict = vars(args)

    # ---------------------------------------------------------
    # ccommand detail

    # RNAseqFromSRA
    if args_dict["subcommand_name"] == "RNAseqFromSRA":
        from api.xuyuxing.resource.ncbi_sra import get_rna_seq_from_sra

        sra_dict = get_rna_seq_from_sra(args.key_word, args.tmp_runinfo_file)

        with open(args.output_file, 'w') as f:
            for i in sra_dict:
                f.write(i + "\n")

    # DownloadGenomeFromNCBI
    elif args_dict["subcommand_name"] == "DownloadGenomeFromNCBI":
        from src.xuyuxing.tools.ncbi_tools import ncbi_genome_downloader
        ncbi_genome_downloader(args)

    # ChooseGoodGenomeFromNCBI
    elif args_dict["subcommand_name"] == "ChooseGoodGenomeFromNCBI":
        from src.xuyuxing.tools.ncbi_tools import ncbi_genome_chooser
        ncbi_genome_chooser(args)



import matplotlib

matplotlib.use('Agg')
import matplotlib.pyplot as plt
import numpy as np
import matplotlib.patches as patches
import matplotlib.path as path


def histogram_style1(data_list, figure_file=None):
    fig = plt.figure()
    ax = fig.add_subplot(111)

    # Fixing random state for reproducibility
    # np.random.seed(19680801)

    # histogram our data with numpy
    # data = np.random.randn(1000)
    n, bins = np.histogram(data_list, bins='auto')

    # get the corners of the rectangles for the histogram
    left = np.array(bins[:-1])
    right = np.array(bins[1:])
    bottom = np.zeros(len(left))
    top = bottom + n
    nrects = len(left)

    nverts = nrects * (1 + 3 + 1)
    verts = np.zeros((nverts, 2))
    codes = np.ones(nverts, int) * path.Path.LINETO
    codes[0::5] = path.Path.MOVETO
    codes[4::5] = path.Path.CLOSEPOLY
    verts[0::5, 0] = left
    verts[0::5, 1] = bottom
    verts[1::5, 0] = left
    verts[1::5, 1] = top
    verts[2::5, 0] = right
    verts[2::5, 1] = top
    verts[3::5, 0] = right
    verts[3::5, 1] = bottom

    barpath = path.Path(verts, codes)
    # patch = patches.PathPatch(barpath, facecolor='green', edgecolor='yellow', alpha=0.5)
    patch = patches.PathPatch(barpath, facecolor='blue', edgecolor="blue", linewidth=0.0)
    ax.add_patch(patch)

    ax.set_xlim(left[0], right[-1])
    ax.set_ylim(bottom.min(), top.max())

    if not figure_file is None:
        plt.savefig(figure_file, dpi=1000)
    plt.show()
    plt.close('all')
    return n, bins


def histogram_style2(data_list, figure_file):
    # fig = plt.figure()
    # ax = fig.add_subplot(111)

    # n, bins, whatever = ax.hist(data_list, 'auto', density=True, histtype='stepfilled', facecolor='b', alpha=0.75)
    n, bins, whatever = plt.hist(data_list, 'auto', histtype='stepfilled')

    # get the corners of the rectangles for the histogram
    # left = np.array(bins[:-1])
    # right = np.array(bins[1:])
    # bottom = np.zeros(len(left))
    # top = bottom + n

    # ax.set_xlim(left[0], right[-1])
    # ax.set_ylim(bottom.min(), top.max())

    plt.savefig(figure_file, dpi=1000)
    plt.show()
    plt.close('all')
    return n, bins


def int_barplot(data_list, figure_file):
    from collections import Counter
    Counter_data = Counter(data_list)
    x, y = [], []
    for i in range(min(Counter_data.keys()), max(Counter_data.keys()) + 1):
        x.append(i)
        y.append(Counter_data[i])

    fig = plt.gcf()
    fig.set_size_inches(10.5, 10.5)
    plt.subplot(111)
    plt.axis([min(x), max(x), min(y), max(y)])
    plt.bar(x, y, width=1)
    # plt.xlabel('time (s)')
    # plt.ylabel('Wild type cmc+')
    ax = plt.gca()
    #ax.spines['left'].set_visible(False)
    ax.spines['top'].set_visible(False)
    ax.spines['right'].set_visible(False)
    #plt.yticks([])
    #plt.xticks([])
    plt.grid(False)

    plt.savefig(figure_file, dpi=1000)

    plt.show()



import xml.etree.cElementTree as etree

def getelements(filename_or_file, tag):
    context = iter(etree.iterparse(filename_or_file, events=('start', 'end')))
    _, root = next(context) # get root element
    for event, elem in context:
        if event == 'end' and elem.tag == tag:
            yield elem
            root.clear() # preserve memory


if __name__ == '__main__':
    import argparse
    import textwrap

    # argument parse
    # ---------------------------------------------------------
    parser = argparse.ArgumentParser(
        prog='GenomeTools',
    )

    subparsers = parser.add_subparsers(
        title='subcommands', dest="subcommand_name")

    # argparse for GenomeSurvey
    parser_a = subparsers.add_parser('GenomeSurvey',
                                     help='easy tools for Genome Survey\n',
                                     description='')

    parser_a.add_argument(
        "genome_size", help="Estimated genome size (based on bp), such as 100000000", type=int)
    parser_a.add_argument("genome_coverage",
                          help="coverage of sequence data of estimated genome size (based on bp), such as 40", type=int)
    parser_a.add_argument(
        "input_file_list", help="a list file containing all fastq file path of input", type=str)
    parser_a.add_argument(
        "-k", "--kmer", help="k-mer for survey (default as 17)", default=17, type=int)
    parser_a.add_argument("-c", "--min_accuracy_rate",
                          help="set min accuracy rate of k-mer, set between 0~0.99, or -1 for unrestrained (default as 0.99)",
                          default=0.99, type=float)
    parser_a.add_argument("-q", "--quality_value",
                          help="set Quality value ascii shift, generally 64 or 33, (default as 33)", default=33,
                          type=int)
    parser_a.add_argument("-t", "--thread_number",
                          help="thread number (default as 1)", default=1, type=int)
    parser_a.add_argument("-p", "--output_prefix", help="set output prefix (default as output)", default="output",
                          type=str)
    parser_a.add_argument("-o", "--output_kmer_frequence_file",
                          help="set whether ouput k-mer frequence file, 0 for no, 1 for yes, default as 0", default=0,
                          type=int)
    parser_a.add_argument("-L", "--maximum_read_length",
                          help="maximum read length, default=150", default=150, type=int)
    parser_a.add_argument("-a", "--begin_ignore_len",
                          help="ignored length of the beginning of a read, default=0", default=0, type=int)
    parser_a.add_argument("-d", "--end_ignore_len",
                          help="ignored length of the end of a read, default=0", default=0, type=int)
    parser_a.add_argument("-print", "--print_info",
                          help="just print parameters for process not running", action='store_true')

    # argparse for AssemblyEvaluator
    parser_a = subparsers.add_parser('AssemblyEvaluator',
                                     help='make a report for assembly accurate by map NGS data\n',
                                     description='')

    parser_a.add_argument('target_assembly_fasta',
                          type=str, help='a fasta file')
    parser_a.add_argument('ngs_1', type=str, help='NGS fastq 1')
    parser_a.add_argument('ngs_2', type=str, help='NGS fastq 2')
    parser_a.add_argument('-o', '--output_file', type=str,
                          help='output file (default: AssemblyEvaluator.txt)', default='AssemblyEvaluator.txt')
    parser_a.add_argument("-t", "--thread_number",
                          help="thread number (default as 1)", default=1, type=int)
    parser_a.add_argument("-d", "--tmp_dir",
                          help="tmp dir for working", default="./", type=str)

    # argparse for ContigDepth
    parser_a = subparsers.add_parser('ContigDepth',
                                     help='get average depth and coverage for each contig\n',
                                     description='')

    parser_a.add_argument('bam_file', type=str, help='sorted bam file')
    parser_a.add_argument('ref_genome', type=str,
                          help='reference genome fasta file')
    parser_a.add_argument(
        "-o", "--output_file", help="the output file (default:ContigDepth.txt)", default="ContigDepth.txt", type=str)

    # argparse for ContaminationFilter
    parser_a = subparsers.add_parser('ContaminationFilter',
                                     help='find contamination contig from genome assembly\n',
                                     description='')

    parser_a.add_argument('split_genome_fasta', type=str,
                          help='blast from organelle hit')
    parser_a.add_argument('taxon_assign_output', type=str,
                          help='survey genome depth results')
    parser_a.add_argument('tax_dir', type=str,
                          help='ncbi taxonomy dir')
    parser_a.add_argument('output_file', type=str, help='output file')

    # argparse for ContigFilter
    parser_a = subparsers.add_parser('ContigFilter',
                                     help='find organelle contig from genome assembly\n',
                                     formatter_class=argparse.RawDescriptionHelpFormatter,
                                     description=textwrap.dedent('''\
                                    example:
                                    -----------------------------------------------
                                    all in one:   
                                    ContigFilter -t 56 -c related_chl_genome.fasta -m related_mit_genome.fasta -g genome.fasta -1 GSS_1.fq.gz -2 GSS_2.fq.gz
                                    
                                    use given rRNA seq
                                    ContigFilter -t 56 -c related_chl_genome.fasta -m related_mit_genome.fasta -r rRNA.unit.seq -g genome.fasta -1 GSS_1.fq.gz -2 GSS_2.fq.gz

                                    use given depth report
                                    ContigFilter -t 56 -c related_chl_genome.fasta -m related_mit_genome.fasta -r rRNA.unit.seq -g genome.fasta -d ContigDepth.txt

                                    use given contamination report
                                    ContigFilter -t 56 -c related_chl_genome.fasta -m related_mit_genome.fasta -r rRNA.unit.seq -g genome.fasta -d ContigDepth.txt

                                    just make plot
                                    ContigFilter -f ContigFilter.txt

                                    '''))

    parser_a.add_argument('-g', '--genome_input_fasta', type=str,
                          help='contig genome fasta file')
    parser_a.add_argument('-1', '--GSS_fq1', type=str,
                          help='genome survey sequence fastq1 file, to get depth value')
    parser_a.add_argument('-2', '--GSS_fq2', type=str,
                          help='genome survey sequence fastq2 file, to get depth value')
    parser_a.add_argument("-t", "--threads",
                          help="thread number (default as 1)", default=1, type=int)
    parser_a.add_argument('-c', '--related_chl_genome', type=str, help='give me a relative species chloroplast genome fasta file. (default: None)',
                          default=None)
    parser_a.add_argument('-m', '--related_mit_genome', type=str, help='give me a relative species mitochondrion genome fasta file. (default: None)',
                          default=None)
    parser_a.add_argument('-r', '--related_nrRNA_seq', type=str, help='give me a relative species nuclear rRNA fasta (unit seq) file. (default: None, will annotated from Ath)',
                          default=None)
    parser_a.add_argument('-d', '--depth_results', type=str, help='give me a results file from ContigDepth',
                          default=None)
    parser_a.add_argument('-p', '--contamination_report', type=str, help='give me a results file from ContaminationFilter',
                          default=None)
    parser_a.add_argument('-f', '--contig_filter_results', type=str, help='give me a results file from ContigFilter, just plot now',
                          default=None)
    parser_a.add_argument('-o', '--work_dir', type=str, help='output dir. (default: ContigFilter)',
                          default='ContigFilter')

    # argparse for FilterAssemblyFile
    parser_a = subparsers.add_parser('FilterAssemblyFile',
                                     help='only keep some contig fragments from assembly file (from juicer)\n',
                                     description='')

    parser_a.add_argument('assembly', type=str, help='assembly file')
    parser_a.add_argument('contig_list', type=str,
                          help='file containing the name of the contig to be keeped')
    parser_a.add_argument('-o', '--output_file', type=str, help='output file (default: new_assembly.txt)',
                          default='new_assembly.txt')

    # argparse for HiC2Fasta
    parser_a = subparsers.add_parser('HiC2Fasta',
                                     help='get chromesome fasta from assembly file (from juicer)\n',
                                     description='')

    parser_a.add_argument('final_fasta', type=str,
                          help='final fasta from 3d-dna, which have contig fragment fasta')
    parser_a.add_argument('assembly', type=str, help='assembly file')
    parser_a.add_argument('output_fasta', type=str, help='output fasta file')

    # argparse for MergeUnanchored
    parser_a = subparsers.add_parser('MergeUnanchored',
                                     help='merge unanchored sequence in a fasta file\n',
                                     description='')

    parser_a.add_argument('merge_info', type=str,
                          help='merge information in a tsv file: 1st col: merged id (chr1), 2ed col: raw id (seq1,seq2,seq3)')
    parser_a.add_argument('input_fasta', type=str, help='input fasta file')
    parser_a.add_argument('output_fasta', type=str, help='output fasta file')

    # argparse for EasyAssemblyPostProcessor
    parser_a = subparsers.add_parser('EasyAssemblyPostProcessor',
                                     help='Easy to use AssemblyPostProcessor tool from PlantTribes\n',
                                     description='')

    parser_a.add_argument('input_fasta', type=str, help='input Trinity.fasta file')
    parser_a.add_argument('output_dir', type=str, help='output dir')
    parser_a.add_argument("-t", "--thread_number",
                          help="thread number (default as 1)", default=1, type=int)



    args = parser.parse_args()
    args_dict = vars(args)

    # ---------------------------------------------------------
    # command detail

    # GenomeSurvey
    if args_dict["subcommand_name"] == "GenomeSurvey":
        from src.xuyuxing.GenomeTools.GenomeSurvey import GenomeSurvey_main
        GenomeSurvey_main(args)

    # AssemblyEvaluator
    elif args_dict["subcommand_name"] == "AssemblyEvaluator":
        from src.xuyuxing.GenomeTools.AssemblyEvaluator import assembly_evaluator_main

        assembly_evaluator_main(args.target_assembly_fasta, args.ngs_1,
                                args.ngs_2, args.output_file, args.thread_number, args.tmp_dir)

    # ContigDepth
    elif args_dict["subcommand_name"] == "ContigDepth":
        """
        qx -pe smp 56 -l h=cn01 bash get_depth.sh nextpolish2_canu.final.rename.fasta Llv ../../../survey/clean/32_lvchun/all.1.cut.fq.trim.pe.gz ../../../survey/clean/32_lvchun/all.2.cut.fq.trim.pe.gz 56

        --bash
        ref_genome=$1
        tag=$2
        fq1=$3
        fq2=$4
        thread=$5

        hisat2-build $ref_genome $ref_genome.hisat2
        hisat2 -x $ref_genome.hisat2 -p $thread -1 $fq1 -2 $fq2 -S $tag.sam

        samtools view -bS -@ $thread -o $tag.bam $tag.sam
        samtools sort -@ $thread -o $tag.sorted.bam $tag.bam
        samtools index -@ $thread $tag.sorted.bam
        rm $tag.sam
        rm $tag.bam

        python ~/python_project/Genome_work_tools/GenAssembly.py ContigDepth -o $tag.GSS.depth $tag.sorted.bam $ref_genome
        --

        class abc():
            pass

        args = abc()
        args.bam_file = '/lustre/home/xuyuxing/Database/Gel/genome/assembly/contig_filter/GSS_vs_Gel.sorted.bam'
        args.ref_genome = '/lustre/home/xuyuxing/Database/Gel/genome/assembly/contig_filter/Gel.genome.v1.0.fasta'
        args.output_file = 
        """

        from src.xuyuxing.GenomeTools.ContigDepth import ContigDepth_main
        ContigDepth_main(args)

    # ContigFilter
    elif args_dict["subcommand_name"] == "ContigFilter":
        from src.xuyuxing.GenomeTools.ContigFilter2 import ContigFilter_main
        ContigFilter_main(args)

    # ContaminationFilter
    elif args_dict["subcommand_name"] == "ContaminationFilter":
        from src.xuyuxing.GenomeTools.ContaminationFilter import ContaminationFilter_main
        ContaminationFilter_main(args)

    # HiC2Fasta
    elif args_dict["subcommand_name"] == "HiC2Fasta":
        from api.xuyuxing.genome.juicer import assembly_to_fasta
        assembly_to_fasta(args.final_fasta, args.assembly, args.output_fasta)

    # AssFileRemove
    elif args_dict["subcommand_name"] == "FilterAssemblyFile":
        from api.xuyuxing.genome.juicer import read_assembly, write_assembly
        from lib.common.fileIO import read_list_file

        # args.contig_list = '/lustre/home/xuyuxing/Database/Lindenbergia/genome/hic_canu/filter/filtered.id'
        # args.assembly = '/lustre/home/xuyuxing/Database/Lindenbergia/genome/hic_canu/filter/Llv.reviewed.rename.assembly.test'
        # args.output_file = '/lustre/home/xuyuxing/Database/Lindenbergia/genome/hic_canu/filter/Llv.reviewed.rename.assembly.test.filter'

        raw_scaffold_dict = read_assembly(args.assembly)
        contig_list = set(read_list_file(args.contig_list))

        filterd_scaffold_dict = {}
        for i in raw_scaffold_dict:
            if len([j for j in raw_scaffold_dict[i]['contig_list'] if j in contig_list]) > 0:
                filterd_scaffold_dict[i] = {'contig_list': [],
                                            'contig_length': [], 'contig_strand': []}
                for c_id, length, strand in zip(raw_scaffold_dict[i]['contig_list'], raw_scaffold_dict[i]['contig_length'], raw_scaffold_dict[i]['contig_strand']):
                    if c_id in contig_list:
                        filterd_scaffold_dict[i]['contig_list'].append(c_id)
                        filterd_scaffold_dict[i]['contig_length'].append(
                            length)
                        filterd_scaffold_dict[i]['contig_strand'].append(
                            strand)

        write_assembly(filterd_scaffold_dict, args.output_file)

    # MergeUnanchored
    elif args_dict["subcommand_name"] == "MergeUnanchored":
        from lib.common.genome.seq_base import read_fasta_by_faidx, FastaRecord, write_fasta
        from lib.common.fileIO import tsv_file_dict_parse

        input_seq_dict = read_fasta_by_faidx(args.input_fasta)
        merge_dict = tsv_file_dict_parse(args.merge_info, fieldnames=[
                                         'new_id', 'raw_id_list'])

        fasta_list = []
        for i in merge_dict:
            new_id = merge_dict[i]['new_id']
            raw_id_list = merge_dict[i]['raw_id_list'].split(",")

            new_seq = ""

            for raw_id in raw_id_list:
                r_rec = input_seq_dict[raw_id]
                r_seq = r_rec.seq

                if not new_seq == "":
                    new_seq = new_seq + "N" * 500

                new_seq = new_seq + r_seq

            fasta_list.append(FastaRecord(seqname=new_id, seq=new_seq))

        write_fasta(fasta_list, args.output_fasta)

    # EasyAssemblyPostProcessor
    elif args_dict["subcommand_name"] == "EasyAssemblyPostProcessor":
        from src.xuyuxing.GenomeTools.EasyPlantTribes import easy_AssemblyPostProcessor_main
        easy_AssemblyPostProcessor_main(args)



from BCBio import GFF
from Bio.Seq import Seq
from Bio.SeqFeature import SeqFeature, FeatureLocation
from Bio.SeqRecord import SeqRecord
from collections import Counter
from lib.common.os import mkdir, multiprocess_running
from lib.xuyuxing.base.common_command import log_print
from lib.common.util import logging_init
from api.xuyuxing.file_parser.fileIO import tsv_file_parse, tsv_file_dict_parse
from lib.common.math.interval import merge_intervals
from lib.xuyuxing.math.set_operating import uniqify
from api.xuyuxing.plot.histogram import int_barplot
from lib.common.genome.seq_base import read_fasta_by_faidx, BioSeq
from pyfaidx import Fasta
import lib.common.sqlite_command as sc
import lib.common.genome.seq_base as sb
import re
import sqlite3
import sys
import time
import os


def FastaByID(ID_list, fasta_file, output_file, log_file, invert_flag=False):
    logger = logging_init("FastaByID", log_file)

    logger.info("Step1: parsing query ID list")
    if output_file is not None:
        F1 = open(output_file, 'w')
    ID_list_short = [re.search('^(\S+)', i).group(1) for i in ID_list]
    logger.info("Step1: finished")

    logger.info("Step2: loading fasta file")
    fasta_dict = Fasta(fasta_file)
    logger.info("Step2: finished")

    num = 0

    if invert_flag is False:
        for query_id in ID_list_short:
            if query_id in fasta_dict:
                num = num + 1
                seq_record = fasta_dict[query_id]
                full_name = seq_record.long_name
                # print(full_name)
                if output_file is not None:
                    F1.write(">%s\n" % full_name)
                    for line in seq_record:
                        F1.write(str(line) + "\n")
                else:
                    print(">%s" % full_name)
                    for line in seq_record:
                        print(line)
    elif invert_flag is True:
        ID_list_short = set(ID_list_short)
        for query_id in fasta_dict.keys():
            if query_id not in ID_list_short:
                num = num + 1
                seq_record = fasta_dict[query_id]
                full_name = seq_record.long_name
                if output_file is not None:
                    F1.write(">%s\n" % full_name)
                    for line in seq_record:
                        F1.write(str(line) + "\n")
                else:
                    print(">%s" % full_name)
                    for line in seq_record:
                        print(line)

    if output_file is not None:
        F1.close()
    logger.info("Found query sequences")
    logger.critical("%d records found in %d queries" %
                    (num, len(ID_list_short)))
    del logger.handlers[:]


def FastaByID_xyx(ID_list, db_fasta_file, output_file, log_file):
    logger = logging_init("FastaByID", log_file)
    logger.info("Loading fasta file")
    if output_file is not None:
        F1 = open(output_file, 'w')
    ID_list_short = [re.search('^(\S+)', i).group(1) for i in ID_list]
    output_dict = []
    for record in sb.read_fasta_big(db_fasta_file, upper=False, log_file=log_file):
        if record.seqname_short() in ID_list_short or record.seqname in ID_list:
            output_dict.append(record)
    output_dict = uniqify(output_dict)
    logger.info("Loaded fasta file")
    logger.info("Finding query sequences")
    for record in output_dict:
        record.wrap()
        if output_file is not None:
            F1.write(">%s\n%s\n" % (record.seqname_short(), record.seq))
        else:
            print(">%s\n%s" % (record.seqname_short(), record.seq))
    if output_file is not None:
        F1.close()
    logger.info("Found query sequences")
    logger.critical("%d records found in %d queries" %
                    (len(output_dict), len(ID_list)))


def FastaByGroup(group_file, db_fasta_file, out_dir, group_name_flag=True):
    if group_name_flag is True:
        group_list = tsv_file_parse(group_file, key_col=1, seq=",")
    else:
        group_list = tsv_file_parse(group_file, seq=",", prefix="Group_")
    group_list_hash = {}
    for group_name in group_list:
        for record_name in group_list[group_name]:
            if record_name in group_list_hash:
                group_list_hash[record_name].append(group_name)
            else:
                group_list_hash[record_name] = [group_name]
            group_list_hash[record_name] = list(
                set(group_list_hash[record_name]))

    group_list_record = {}
    for record in sb.read_fasta_big(db_fasta_file, upper=False):
        record_name = record.seqname_short()
        if record_name in group_list_hash:
            for group_name in group_list_hash[record_name]:
                if group_name in group_list_record:
                    group_list_record[group_name].append(record)
                else:
                    group_list_record[group_name] = [record]
                group_list_record[group_name] = uniqify(
                    group_list_record[group_name])

    mkdir(out_dir)

    for group_name in group_list_record:
        with open(out_dir + "/" + group_name + ".fasta", 'w') as f:
            for record in group_list_record[group_name]:
                f.write(">%s\n%s\n" % (record.seqname_short(), record.seq))


def subfasta(db_fasta_file, search_string, input_file, RNA_flag, output_file, bed_file):
    out_dir = {}
    if search_string is not None:
        refseq, start, end, strand = re.search(
            r'^(\S+):(\d+)-(\d+):(\S)$', search_string).groups()
        out = sb.sub_fasta(db_fasta_file, refseq, int(
            start), int(end), strand, RNA_flag)
        out_dir[search_string] = out

    if input_file is not None:
        if not bed_file:
            sub_table = tsv_file_parse(
                input_file, key_col=1, fields="2,3,4,5", seq=",")
        else:
            sub_table = tsv_file_parse(
                input_file, key_col=4, fields="1,2,3,6", seq="\t")
        print(sub_table)
        if RNA_flag:
            for i in sub_table.keys():
                sub_table[i] = sub_table[i] + (True,)
            out_dir = sb.sub_fasta_many(db_fasta_file, sub_table)
        else:
            for i in sub_table.keys():
                sub_table[i] = sub_table[i] + (False,)
            out_dir = sb.sub_fasta_many(db_fasta_file, sub_table)

    if output_file is not None:
        F1 = open(output_file, 'w')

    for record_name in out_dir:
        record = out_dir[record_name]
        record.wrap()
        if output_file is not None:
            F1.write(">%s\n%s" % (record.seqname_short(), record.seq))
        else:
            print(">%s\n%s" % (record.seqname_short(), record.seq))
    if output_file is not None:
        F1.close()


def extract_longest_isofrom_from_gene_in_trinity(trinity_output):
    longest_dict = {}
    for record in sb.read_fasta_big(trinity_output):
        cluster_id, gene_id, isoform_id = re.match(r'^(\S+c\d+)_(g\d+)_(i\d+)$',
                                                   record.seqname_short()).groups()
        gene_name = cluster_id + "_" + gene_id
        if not gene_name in longest_dict:
            longest_dict[gene_name] = record
        elif len(longest_dict[gene_name].seq) < len(record.seq):
            longest_dict[gene_name] = record
    return longest_dict


def split_fasta(input_file, output_dir, outpre, contig_model=False, unit_num=None, file_size=None):
    mkdir(output_dir, True)

    record_dict = sb.read_fasta_by_faidx(input_file)
    if contig_model:
        for i in record_dict:
            record_tmp = record_dict[i]
            with open(output_dir + "/" + record_tmp.seqname_short() + ".fa", 'w') as f:
                f.write(">%s\n%s" % (record_tmp.seqname, record_tmp.seq))
    elif unit_num:
        all_used = 0
        num_used = 0
        file_num = 0
        F1 = open(output_dir + "/%s%d.fa" % (outpre, file_num), 'w')

        for record_id in record_dict:
            record = record_dict[record_id]
            num_used = num_used + 1
            F1.write(">%s\n%s\n" % (record.seqname, record.seq))
            all_used += 1
            if num_used >= unit_num:
                F1.close()
                file_num = file_num + 1
                if all_used < len(record_dict):
                    F1 = open(output_dir + "/%s%d.fa" % (outpre, file_num), 'w')
                num_used = 0

        try:
            F1.close()
        except:
            pass

    elif file_size:
        sum_size = 0
        file_num = 0
        F1 = open(output_dir + "/%s%d.fa" % (outpre, file_num), 'w')

        for record_id in record_dict:
            record = record_dict[record_id]
            if sum_size == 0 and record.len() >= file_size:
                new_file_flag = True
                add_to_new_file = False
                F1.write(">%s\n%s\n" % (record.seqname, record.seq))
                sum_size += record.len()
                # print(sum_size)
            elif sum_size + record.len() >= file_size:
                new_file_flag = True
                add_to_new_file = True
                sum_size += record.len()
                # print(sum_size)
            else:
                new_file_flag = False
                add_to_new_file = False
                F1.write(">%s\n%s\n" % (record.seqname, record.seq))
                sum_size += record.len()

            if new_file_flag:
                F1.close()
                file_num = file_num + 1
                F1 = open(output_dir + "/%s%d.fa" % (outpre, file_num), 'w')
                sum_size = 0
                if add_to_new_file:
                    F1.write(">%s\n%s\n" % (record.seqname, record.seq))
                    sum_size += record.len()

        F1.close()


def FastqByID(ID_list, db_file, output_prefix):
    def output_result(db_file, ID_list_tmp, paired_flag, output_prefix, frist_write):
        if paired_flag:
            record_tuple_list_f = sc.sqlite_select_by_a_key(db_file, "record", "fseqname_short",
                                                            tuple(ID_list_tmp))
            record_tuple_list_r = sc.sqlite_select_by_a_key(db_file, "record", "rseqname_short",
                                                            tuple(ID_list_tmp))
            record_tuple_list = list(
                set(record_tuple_list_f + record_tuple_list_r))
        else:
            record_tuple_list = sc.sqlite_select_by_a_key(db_file, "record", "fseqname_short",
                                                          tuple(ID_list_tmp))
            record_tuple_list = list(set(record_tuple_list))

        if output_prefix is not None:
            if paired_flag:
                if frist_write:
                    F1 = open(output_prefix + "_1.fq", 'w')
                    F2 = open(output_prefix + "_2.fq", 'w')
                else:
                    F1 = open(output_prefix + "_1.fq", 'a')
                    F2 = open(output_prefix + "_2.fq", 'a')
            else:
                if frist_write:
                    F1 = open(output_prefix + ".fq", 'w')
                else:
                    F1 = open(output_prefix + ".fq", 'a')

        for record in record_tuple_list:
            if output_prefix is not None:
                if paired_flag:
                    record1_seqname_short, record1_seqname, record1_seqs, record1_quality, record2_seqname_short, record2_seqname, record2_seqs, record2_quality = record
                    F1.write(
                        "@%s\n%s\n+%s\n%s\n" % (
                            record1_seqname, record1_seqs, record1_seqname, record1_quality))
                    F2.write(
                        "@%s\n%s\n+%s\n%s\n" % (
                            record2_seqname, record2_seqs, record2_seqname, record2_quality))
                else:
                    record1_seqname_short, record1_seqname, record1_seqs, record1_quality = record
                    F1.write(
                        "@%s\n%s\n+%s\n%s\n" % (
                            record1_seqname, record1_seqs, record1_seqname, record1_quality))
            else:
                if paired_flag:
                    record1_seqname_short, record1_seqname, record1_seqs, record1_quality, record2_seqname_short, record2_seqname, record2_seqs, record2_quality = record
                    print("F: @%s\nF: %s\nF: +%s\nF: %s\n" % (
                        record1_seqname, record1_seqs, record1_seqname, record1_quality))
                    print("R: @%s\nR: %s\nR: +%s\nR: %s\n" % (
                        record2_seqname, record2_seqs, record2_seqname, record2_quality))
                else:
                    record1_seqname_short, record1_seqname, record1_seqs, record1_quality = record
                    print("@%s\n%s\n+%s\n%s\n" % (
                        record1_seqname, record1_seqs, record1_seqname, record1_quality))

        if output_prefix is not None:
            if paired_flag:
                F1.close()
                F2.close()
            else:
                F1.close()

        return len(record_tuple_list)

    log_print("Begin: Extract fastq files from sqlite3 db")
    start_time = time.time()

    conn = sqlite3.connect(db_file)
    content = conn.execute("SELECT * FROM info").fetchall()[0]
    conn.close()

    if content[0] == 'paired_end':
        paired_flag = True
    else:
        paired_flag = False

    num = 0
    frist_write = True
    ID_list_tmp = []
    output_num = 0
    for ID in ID_list:
        ID_list_tmp.append(ID)
        num = num + 1
        if num % 10000 == 0:
            output_num = output_num + output_result(db_file, ID_list_tmp, paired_flag, output_prefix,
                                                    frist_write)
            frist_write = False
            ID_list_tmp = []

        round_time = time.time()
        if round_time - start_time > 10:
            log_print("\tparsed: %d" % (num))
            start_time = round_time

    if len(ID_list_tmp) > 0:
        output_num = output_num + \
            output_result(db_file, ID_list_tmp, paired_flag,
                          output_prefix, frist_write)
        ID_list_tmp = []
        log_print("\tparsed: %d" % (num))

    log_print("End: Extract fastq files from sqlite3 db")

    return output_num, len(ID_list)


def store_fastq_record_into_sqlite(fq1, fq2, output_file, gzip_flag):
    log_print("Begin: convert fastq files to sqlite3 db")
    start_time = time.time()

    if fq2 is None:
        table_columns_dict = {
            "info": ["end_type", "ffq_file_name"],
            "record": ["fseqname_short", "fseqname", "fseqs", "fquality"]
        }
        sc.init_sql_db_many_table(output_file, table_columns_dict)
        sc.insert_one_record_to_sql_table(
            ("single_end", fq1), table_columns_dict["info"], output_file, "info")
    else:
        table_columns_dict = {
            "info": ["end_type", "ffq_file_name", "rfq_file_name"],
            "record": ["fseqname_short", "fseqname", "fseqs", "fquality", "rseqname_short", "rseqname", "rseqs",
                       "rquality"]
        }
        sc.init_sql_db_many_table(output_file, table_columns_dict)
        sc.insert_one_record_to_sql_table(("paired_end", fq1, fq2), table_columns_dict["info"], output_file,
                                          "info")

    num = 0
    record_tmp_dict = []
    for records in sb.read_fastq_big(fq1, fq2, gzip_flag):
        if fq2 is not None:
            record1 = records[0]
            record2 = records[1]
            record_tmp_dict.append(
                (
                    record1.seqname_short(), record1.seqname, record1.seq, record1.quality,
                    record2.seqname_short(),
                    record2.seqname, record2.seq, record2.quality))
        else:
            record1 = records[0]
            record_tmp_dict.append(
                (record1.seqname_short(), record1.seqname, record1.seq, record1.quality))

        num = num + 1

        if num % 10000 == 0:
            sc.sqlite_write(record_tmp_dict, output_file,
                            "record", table_columns_dict["record"])
            record_tmp_dict = []

        round_time = time.time()
        if round_time - start_time > 10:
            log_print("\tparsed: %d" % (num))
            start_time = round_time

    if len(record_tmp_dict) > 0:
        sc.sqlite_write(record_tmp_dict, output_file,
                        "record", table_columns_dict["record"])
        record_tmp_dict = []
        log_print("\tparsed: %d" % (num))

    conn = sqlite3.connect(output_file)
    if fq2 is not None:
        conn.execute(
            "CREATE UNIQUE INDEX fseqname_index on record (fseqname_short)")
        conn.execute(
            "CREATE UNIQUE INDEX rseqname_index on record (rseqname_short)")
    else:
        conn.execute(
            "CREATE UNIQUE INDEX fseqname_index on record (fseqname_short)")
    conn.close()

    log_print("End: convert fastq files to sqlite3 db")


def FastaRename(ID_dict, db_fasta_file, output_file):
    """
    ID_dict =
    db_fasta_file =
    output_file:
    """

    if output_file is not None:
        F1 = open(output_file, 'w')

    output_dict = []
    for record in sb.read_fasta_big(db_fasta_file, upper=False):
        if record.seqname_short() in ID_dict:
            record.newname = ID_dict[record.seqname_short()][0]
            output_dict.append(record)
        elif record.seqname in ID_dict:
            record.newname = ID_dict[record.seqname][0]
            output_dict.append(record)

    output_dict = uniqify(output_dict)

    for record in output_dict:
        record.wrap()
        if output_file is not None:
            F1.write(">%s\n%s" % (record.newname, record.seq))
        else:
            print(">%s\n%s" % (record.newname, record.seq))
    if output_file is not None:
        F1.close()

    return len(output_dict), len(ID_dict)


def FastaRename_faidx(ID_dict, db_fasta_file, output_file):
    # if output_file is not None:
    #     F1 = open(output_file, 'w')

    db_file_dict = sb.read_fasta_by_faidx(db_fasta_file)
    num = 0
    for old_name in ID_dict:
        new_name = ID_dict[old_name][0]
        if old_name in db_file_dict:
            num = num + 1
            record = db_file_dict[old_name]
            record.seqname = new_name
            # record.wrap()
            # print("wrap done")
            if output_file is not None:
                record.write_to_file(output_file)
                # F1.write(">%s\n%s" % (new_name, record.seq))
            else:
                print(">%s\n%s" % (new_name, record.seq))
        else:
            print('failed to find %s' % old_name)
    #
    # if output_file is not None:
    #     F1.close()

    return num, len(ID_dict)


# main

# FastaByID
def FastaByID_main(args):

    db_fasta_file = args.db_fasta_file
    output_file = args.output_file
    log_file = args.log_file
    old_way = args.old_way

    ID_file = args.ID_file
    ID_list = args.ID_list
    if ID_file:
        ID_list = tsv_file_parse(ID_file, key_col=1)
        ID_list = ID_list.keys()
    elif ID_list:
        ID_list = ID_list.split(",")

    if not old_way:
        FastaByID(ID_list, db_fasta_file, output_file,
                  log_file, args.invert_flag)
    else:
        FastaByID_xyx(ID_list, db_fasta_file, output_file, log_file)

# FastaByGroup


def FastaByGroup_main(args):
    group_file = args.group_file
    db_fasta_file = args.db_fasta_file
    out_dir = args.out_dir
    group_name_flag = args.name_flag
    FastaByGroup(group_file, db_fasta_file, out_dir, group_name_flag)

# subfasta


def subfasta_main(args):
    db_fasta_file = args.db_fasta_file
    search_string = args.search_string
    input_file = args.input_file
    RNA_flag = args.RNA
    output_file = args.output_file

    if input_file is None and search_string is None:
        raise ValueError("input_file or search_string need at least one")

    subfasta(db_fasta_file, search_string, input_file,
             RNA_flag, output_file, args.input_a_bed_file)

# PreExtractNr


def PreExtractNr_main(args):
    nr_fasta_file = args.nr_fasta_file
    sql3_db_file = args.sql3_db_file
    sb.PreExtractNr(nr_fasta_file, sql3_db_file)

# ExtractNr


def ExtractNr_main(args):

    db_fasta_file = args.db_fasta_file
    output_file = args.output_file
    long_name = args.long_name

    ID_file = args.ID_file
    ID_list = args.ID_list
    if ID_file:
        ID_list = tsv_file_parse(ID_file, key_col=1)
        ID_list = ID_list.keys()
    elif ID_list:
        ID_list = ID_list.split(",")

    found_num, want_num = sb.ExtractNr(db_fasta_file, tuple(
        ID_list), output_file, short_name=long_name)
    print("%d records found in %d queries" % (found_num, want_num))

# TrinityGene


def TrinityGene_main(args):
    Trinity_output = args.Trinity_output
    Trinity_gene = args.Trinity_gene
    output = extract_longest_isofrom_from_gene_in_trinity(Trinity_output)
    with open(Trinity_gene, "w") as f:
        for i in output:
            i = output[i]
            i.wrap(60)
            f.write(">%s\n%s" % (i.seqname, i.seq))

# GenerateTrinityGeneTransMap


def GenerateTrinityGeneTransMap_main(args):
    Trinity_output = args.Trinity_output
    Trinity_gene = args.gene_map_file
    with open(Trinity_output, 'r') as f:
        with open(Trinity_gene, 'w') as w:
            for each_line in f:
                each_line = each_line.strip()
                record_head = re.match(r"^>", each_line)
                if record_head:
                    seqname = re.sub(r'^>', '', each_line)
                    name_short = re.search('^(\S+)', seqname).group(1)
                    gene_name = re.sub(r'\_i\d+$', '', name_short)
                    w.write("%s\t%s\n" % (gene_name, name_short))

# SeqLength


def SeqLength_main(args):
    seq_file = args.seq_file
    output_file = args.output_file
    stat = args.stat
    picture = args.picture
    fq_flag = args.fq_flag

    if output_file is not None:
        F1 = open(output_file, 'w')

    seq_len_list = []
    if not fq_flag:
        seq_file_gen = sb.read_fasta_big(seq_file, upper=False)
    else:
        seq_file_gen = sb.read_fastq_big(seq_file)

    for record in seq_file_gen:
        if fq_flag:
            record = record[0]
        if stat is False:
            if output_file is not None:
                F1.write("%s\t%d\n" %
                         (record.seqname_short(), len(record.seq)))
            else:
                print("%s\t%s" % (record.seqname_short(), len(record.seq)))
        else:
            seq_len_list.append(len(record.seq))

    if stat:
        if not picture == "":
            int_barplot(seq_len_list, picture)
        else:
            from collections import Counter

            Counter_data = Counter(seq_len_list)
            for i in range(min(Counter_data.keys()), max(Counter_data.keys()) + 1):
                if output_file is not None:
                    F1.write("%d\t%d\n" % (i, Counter_data[i]))
                else:
                    print("%d\t%d" % (i, Counter_data[i]))

    if output_file is not None:
        F1.close()

# SplitFasta


def SplitFasta_main(args):
    # split_fasta(input_file, unit_num, output_dir, outpre, args.contig_model)

    split_fasta(args.fasta_file, os.getcwd(), args.output_prefix, contig_model=args.contig_model,
                unit_num=args.records, file_size=args.size)

# AlnStat


def AlnStat_main(args):
    aln_file = args.aln_file
    query = args.query
    output_file = args.output_file

    record_dict, seqname_list = sb.read_fasta(aln_file)
    col_num = len(record_dict[seqname_list[0]].seq)
    seq_num = len(seqname_list)
    count_dict = {}
    for i in range(0, col_num):
        count_dict[i] = dict(Counter([record_dict[seqname].seq[i]
                                      for seqname in record_dict]))

    if query is None:
        with open(output_file, "w") as f:
            for site in count_dict:
                highest = sorted(
                    count_dict[site], key=lambda x: count_dict[site][x], reverse=True)[0]
                if not highest == "-":
                    ratio_tmp = float(
                        count_dict[site][highest]) / float(seq_num)
                else:
                    ratio_tmp = 0.0
                f.write("%d\t%s\n" % (site, ratio_tmp))
    else:
        with open(output_file, "w") as f:
            num = 0
            for site in count_dict:
                if record_dict[query].seq[site] == "-":
                    continue
                num = num + 1
                highest = sorted(
                    count_dict[site], key=lambda x: count_dict[site][x], reverse=True)[0]
                if not highest == "-":
                    ratio_tmp = float(
                        count_dict[site][highest]) / float(seq_num)
                else:
                    ratio_tmp = 0.0
                f.write("%d\t%s\n" % (num, ratio_tmp))

# FastqByID


def FastqByID_main(args):
    db_file = args.db_file
    ID_list = args.ID_list
    ID_file = args.ID_file
    output_prefix = args.output_prefix

    if ID_file:
        ID_list = tsv_file_parse(ID_file, key_col=1)
        ID_list = ID_list.keys()
    elif ID_list:
        ID_list = ID_list.split(",")

    ID_list_used = []
    for i in ID_list:
        ID_list_used.append(re.search('^(\S+)', i).group(1))

    found_num, want_num = FastqByID(ID_list_used, db_file, output_prefix)
    print("%d records found in %d queries" % (found_num, want_num))

# PreFastqByID


def PreFastqByID_main(args):
    fastq_1 = args.fastq_1
    fastq_2 = args.fastq_2
    output_file = args.output_file
    gzip_flag = args.gzip_flag

    store_fastq_record_into_sqlite(fastq_1, fastq_2, output_file, gzip_flag)

# FastaRename


def FastaRename_main(args):
    ID_map_file = args.ID_map_file
    db_fasta_file = args.db_fasta_file
    output_file = args.output_file

    if args.seq_prefix is not None:
        num = 0
        rename_map = []
        with open(args.output_file, 'w') as f:
            for record in sb.read_fasta_big(db_fasta_file, upper=False):
                record.wrap()
                new_name = args.seq_prefix + str(num)
                f.write(">%s\n%s" % (new_name, record.seq))
                rename_map.append((record.seqname, new_name))
                num = num + 1
        with open(args.output_file + ".rename.map", 'w') as f:
            for i in rename_map:
                f.write("%s\t%s\n" % (i[1], i[0]))

    else:
        # ID_map_file = "/lustre/home/xuyuxing/Work/miRNA/Cau_two_assembly/remove/remove/Cucumis_Cau/Cau+Cucumis.del.map"
        ID_dict = tsv_file_parse(ID_map_file, key_col=1, fields="2")
        if args.old_way:
            found_num, want_num = FastaRename(
                ID_dict, db_fasta_file, output_file)
        else:
            found_num, want_num = FastaRename_faidx(
                ID_dict, db_fasta_file, output_file)
        print("%d records found in %d queries" % (found_num, want_num))

# FilterByLen


def FilterByLen_main(args):
    seq_file = args.seq_file
    output_file = args.output_file
    condition_string = args.condition_string

    with open(output_file, "w") as f:
        for record in sb.read_fasta_big(seq_file, upper=False):
            seq = record.seqs_length()
            record.wrap()
            # print(seq,condition_string,eval(condition_string))
            if eval(condition_string):
                f.write(">%s\n%s\n" % (record.seqname, record.seq))

# FastaToSQL


def FastaToSQL_main(args):
    fasta_file = args.fasta_file
    sqlite_file = args.sqlite_file
    log_file = args.log_file
    gzip_flag = args.gzip_flag

    sb.read_fasta_to_sqlite(fasta_file, sqlite_file, gzip_flag, log_file)

# FastaFormatClean


def FastaFormatClean_main(args):
    raw_fasta_file = args.raw_fasta_file
    output_fasta_file = args.output_fasta_file

    seq_type = args.seq_type
    if seq_type == 'nucl':
        seq_type = 0b0001
    elif seq_type == 'prot':
        seq_type = 0b0000

    dsa_flag = args.degenerate_sites_allowed
    if dsa_flag:
        dsa_flag = 0b0010

    ga_flag = args.gap_allowed
    if ga_flag:
        ga_flag = 0b0100

    tsa_flag = args.translation_stop_allowed
    if tsa_flag:
        tsa_flag = 0b1000

    wrap = args.wrap
    replace_flag = args.replace_flag
    full_name_flag = args.full_name_flag
    log_file = args.log_file

    bit_field = seq_type + dsa_flag + ga_flag + tsa_flag

    logger = logging_init("FastaFormatClean", log_file)

    logger.info("Start: parsing fasta file")

    if bit_field == 0b0111:
        all_allowed = sb.FastaRecord.nucl + sb.FastaRecord.any_nucl + \
            sb.FastaRecord.nucl_degenerate + sb.FastaRecord.gap_site
        safe_remove = []
        safe_replace = sb.FastaRecord.any_nucl
    elif bit_field == 0b0011:
        all_allowed = sb.FastaRecord.nucl + \
            sb.FastaRecord.any_nucl + sb.FastaRecord.nucl_degenerate
        safe_remove = sb.FastaRecord.gap_site
        safe_replace = sb.FastaRecord.any_nucl
    elif bit_field == 0b0101:
        all_allowed = sb.FastaRecord.nucl + \
            sb.FastaRecord.any_nucl + sb.FastaRecord.gap_site
        safe_remove = []
        safe_replace = sb.FastaRecord.any_nucl
    elif bit_field == 0b0001:
        all_allowed = sb.FastaRecord.nucl + sb.FastaRecord.any_nucl
        safe_remove = sb.FastaRecord.gap_site
        safe_replace = sb.FastaRecord.any_nucl
    elif bit_field == 0b0000:
        all_allowed = sb.FastaRecord.prot + sb.FastaRecord.any_prot
        safe_remove = sb.FastaRecord.gap_site + sb.FastaRecord.stop_site
        safe_replace = sb.FastaRecord.any_prot
    elif bit_field == 0b0100:
        all_allowed = sb.FastaRecord.prot + \
            sb.FastaRecord.any_prot + sb.FastaRecord.gap_site
        safe_remove = sb.FastaRecord.stop_site
        safe_replace = sb.FastaRecord.any_prot
    elif bit_field == 0b1100:
        all_allowed = sb.FastaRecord.prot + sb.FastaRecord.any_prot + \
            sb.FastaRecord.gap_site + sb.FastaRecord.stop_site
        safe_remove = []
        safe_replace = sb.FastaRecord.any_prot
    elif bit_field == 0b1000:
        all_allowed = sb.FastaRecord.prot + \
            sb.FastaRecord.any_prot + sb.FastaRecord.stop_site
        safe_remove = sb.FastaRecord.gap_site
        safe_replace = sb.FastaRecord.any_prot
    elif bit_field == 0b0010:
        all_allowed = sb.FastaRecord.prot + \
            sb.FastaRecord.any_prot
        safe_remove = sb.FastaRecord.gap_site
        safe_replace = sb.FastaRecord.stop_site
    else:
        raise ValueError("Bad input flag")

    with open(output_fasta_file, 'w') as f:
        for record in sb.read_fasta_big(raw_fasta_file, upper=True, log_file=log_file):
            logger.info("parsing %s with code %d" %
                        (record.seqname_short(), bit_field))
            record.no_wrap()
            new_seq = record.seq
            char_site = set(new_seq)
            bad_flag = False
            for i in char_site:
                if i in all_allowed:
                    continue
                elif i in safe_remove:
                    if i == "*":
                        new_seq = re.sub("\*", "", new_seq)
                    else:
                        new_seq = re.sub(i, "", new_seq)
                else:
                    if replace_flag:
                        if i == "*":
                            new_seq = re.sub("\*", "", new_seq)
                        else:
                            new_seq = re.sub(i, safe_replace[0], new_seq)
                    else:
                        logger.error("%s have bad site %s" %
                                     (record.seqname_short(), i))
                        bad_flag = True

            if bad_flag is True:
                continue

            record.seq = new_seq
            record.wrap(wrap)

            if full_name_flag:
                new_name = record.seqname
            else:
                new_name = record.seqname_short()

            f.write(">%s\n%s" % (new_name, record.seq))

# ReComp


def ReComp_main(args):
    with open(args.output_fasta_file, 'w') as f:
        for record in sb.read_fasta_big(args.input_fasta_file, upper=True):
            record.no_wrap()
            new_seq = record.seq

            if args.only_reverse:
                new_seq = sb.reverse_seq(new_seq)
                new_name = record.seqname + "_reverse"
            elif args.only_complement:
                new_seq = sb.complement_seq(new_seq)
                new_name = record.seqname + "_complement"
            else:
                new_seq = sb.reverse_complement(new_seq)
                new_name = record.seqname + "_reverse_complement"
            record.seq = new_seq
            record.wrap(60)

            f.write(">%s\n%s" % (new_name, record.seq))

# FastqSize


def FastqSize_main(args):
    count_read1 = 0
    count_read1_base = 0
    count_read2 = 0
    count_read2_base = 0
    for records in sb.read_fastq_big(args.fastq_1, args.fastq_2, args.gzip_flag):
        if args.fastq_2 is not None:
            record1 = records[0]
            record2 = records[1]
            count_read1 = count_read1 + 1
            count_read2 = count_read2 + 1
            count_read1_base = count_read1_base + len(record1.seq)
            count_read2_base = count_read2_base + len(record2.seq)
        else:
            record1 = records[0]
            count_read1 = count_read1 + 1
            count_read1_base = count_read1_base + len(record1.seq)

    if args.fastq_2 is not None:
        print("The fastq is paired:\n%s\t%s\n\n\tfastq_1\tfastq_2\tsum" %
              (args.fastq_1, args.fastq_2))
        print("base_num\t%d\t%d\t%d" % (count_read1_base,
                                        count_read2_base, count_read1_base + count_read2_base))
        print("count_num\t%d\t%d\t%d" %
              (count_read1, count_read2, count_read1 + count_read2))
    else:
        print("The fastq is single:\n%s\n\n\tfastq" % args.fastq_1)
        print("base_num\t%d" % (count_read1_base))
        print("count_num\t%d" % (count_read1))

# FastaStats


def FastaStats_main(args):
    """
    class abc():
        pass

    args = abc()
    args.old_way = False
    args.fasta_file = '/lustre/home/xuyuxing/Database/Cuscuta/Cau/genomev1.1/Cuscuta.genome.v1.1.fasta'


    """
    from lib.xuyuxing.math.stats import n50_stats
    import lib.common.genome.seq_base as sb
    from pyfaidx import Fasta

    if args.old_way is False:
        len_list = (len(i) for i in Fasta(args.fasta_file))
    else:
        len_list = (
            record.seqs_length for record in sb.read_fasta_big(args.fasta_file))

    percent_stats, index_stats_dict, count_sum, sum_len, min_len, max_len = n50_stats(
        len_list)

    output_string = "%s\n%s\t%s\t%s\t%s\n" % (
        args.fasta_file, "Nx", "Size", "Count_Sum", "Base_Sum")
    for i in index_stats_dict:
        output_string = output_string + "N%d\t%d\t%d\t%d\n" % (
            i, index_stats_dict[i]['length'], index_stats_dict[i]['count'], index_stats_dict[i]['base'])

    output_string = output_string + "\nN-perc\tSize\n"

    args.index_list = [10, 20, 30, 40, 50, 60, 70, 80, 90, 100]
    for i in range(0, len(percent_stats)):
        output_string = output_string + \
            "%d\t%.2f\n" % (args.index_list[i], percent_stats[i])

    output_string = output_string + \
        "\nTotal Length/Count\t%d\t%d\n" % (sum_len, count_sum)
    output_string = output_string + "Maximum Length\t%d\n" % max_len
    output_string = output_string + "Minimum Length\t%d\n" % min_len

    print(output_string)

    if not args.output_file is None:
        with open(args.output_file, 'w') as f:
            f.write(output_string)

# FragmentGenome


def FragmentGenome_main(args):
    genome_file = args.genome_file
    output_file = args.output_file
    step = args.step
    length = args.length
    Consider_scaffold = args.Consider_scaffold
    shift_start = args.shift_start

    with open(output_file, 'w') as f:
        for i in sb.read_fasta_big(genome_file):
            record_tmp = i
            seq_name = record_tmp.seqname_short()
            sequence = record_tmp.seq
            # print(shift_start)
            for start, sub_seq in sb.scaffold_cutting(sequence, step=step, length=length,
                                                      Consider_scaffold=Consider_scaffold, shift_start=shift_start):
                # print(start)
                end = len(sub_seq) + start - 1
                sub_seq_name = seq_name + "_" + str(start) + "-" + str(end)

                if sb.sequence_entropy(sub_seq, 3) > args.entropy_threshold:

                    f.write(">%s\n%s\n" % (sub_seq_name, sub_seq))

# ReGetContig

def contig_range(SeqRecord_input):
    sequence = str(SeqRecord_input.seq)

    num = 1
    n_site = []
    for i in sequence:
        if i != "N":
            n_site.append((num, num))
        num = num + 1
    contig_range_output = merge_intervals(n_site, True)

    SeqRecord_input.features = []

    num = 0
    for contig_range_tmp in contig_range_output:
        qualifiers = {
            "source": "ReGetContig",
            "ID": "%s_%d" % (SeqRecord_input.id, num)
        }
        top_feature = SeqFeature(FeatureLocation(contig_range_tmp[0] - 1, contig_range_tmp[1]),
                                    type="contig", qualifiers=qualifiers)
        SeqRecord_input.features.append(top_feature)
        num = num + 1

    return SeqRecord_input

def ReGetContig_main(args):
    """
    class abc():
        pass

    args=abc()

    args.scaff_file = "/lustre/home/xuyuxing/Database/Cuscuta/Csp/Csp.fa"
    args.threads = 5
    args.output_gff3 = "/lustre/home/xuyuxing/Work/Csp/HiC/contig.gff3"
    args.output_fasta = "/lustre/home/xuyuxing/Work/Csp/HiC/contig.fasta"
    """

    scaff_dir = Fasta(args.scaff_file)

    SeqRecord_list = []

    for scaff in scaff_dir:
        scaff_seq = Seq(str(scaff))
        SeqRecord_tmp = SeqRecord(scaff_seq, scaff.name)
        SeqRecord_list.append((SeqRecord_tmp,))

    cmd_result = multiprocess_running(
        contig_range, SeqRecord_list, args.threads, silence=True)

    with open(args.output_gff3, 'w') as f:
        for i in cmd_result:
            GFF.write([cmd_result[i]['output']], f)

    with open(args.output_fasta, 'w') as f:
        for i in cmd_result:
            SeqRecord_tmp = cmd_result[i]['output']

            record_feature = SeqRecord_tmp.features
            for contig_feature in record_feature:
                start = int(contig_feature.location.start)
                end = int(contig_feature.location.end)
                ID = contig_feature.qualifiers['ID'][0]
                contig_seq = SeqRecord_tmp.seq[start:end]

                contig_record = SeqRecord(contig_seq, id=ID,
                                          description="%s:%d-%d" % (SeqRecord_tmp.id, start + 1, end))
                f.write(contig_record.format("fasta"))
                # SeqIO.write([contig_record], args.output_fasta, "fasta")

# GeneStructure


def GeneStructure_main(args):
    """
    class abc():
        pass

    args = abc()
    args.gff_file = "/lustre/home/xuyuxing/Database/Plant_genome/raw_data/Gastrodia_elata/GWHAAEX00000000.gff"
    args.genome_fasta_file = "/lustre/home/xuyuxing/Database/Plant_genome/raw_data/Gastrodia_elata/GWHAAEX00000000.genome.fasta"
    args.gene_id = "evm.TU.scaffold_0.420"
    args.output_file = None
    """

    if args.output_file is None:
        file = sys.stdout
    else:
        file = open(args.output_file, 'w')

    genome_dict = read_fasta_by_faidx(args.genome_fasta_file)

    in_file = args.gff_file

    with open(in_file, 'r') as in_handle:
        get_flag = False
        for rec in GFF.parse(in_handle):
            for gene in rec.features:
                if gene.id == args.gene_id:
                    get_flag = True
                    break
            if get_flag:
                break

    contig_record = genome_dict[rec.id]
    sub_seq_tmp = contig_record.sub(1, 100, '+', RNA=False)

    if get_flag:
        if gene.strand == 1:
            strand = "+"
        else:
            strand = "-"

        file.write("# Find gene: %s, on contig %s, strand %s\n" %
                   (gene.id, rec.id, strand))
        file.write("# There are %d mRNA in gene\n\n" % len(gene.sub_features))

        num = 1
        for mRNA in gene.sub_features:
            file.write("mRNA %d (ID: %s): \n\n" % (num, mRNA.id))
            file.write("total seq: \n")
            start = int(mRNA.location.start) + 1
            end = int(mRNA.location.end)
            seq_tmp = contig_record.sub(start, end, strand, RNA=False)
            file.write(">%s:%d-%d:%s\n" % (rec.id, start, end, strand))
            file.write("%s\n\n" % seq_tmp)

            # upstream_range 3k
            file.write("upstream range (3k): \n")
            if strand == "+":
                start = int(mRNA.location.start) + 1 - 3000
                end = int(mRNA.location.start) + 1 - 1
            else:
                end = int(mRNA.location.end) + 3000
                start = int(mRNA.location.end) + 1
            seq_tmp = contig_record.sub(start, end, strand, RNA=False)
            file.write(">%s:%d-%d:%s\n" % (rec.id, start, end, strand))
            file.write("%s\n\n" % seq_tmp)

            # mRNA sub feature
            if strand == "+":
                print_sub_feature = sorted([i for i in mRNA.sub_features if i.type != 'exon'],
                                           key=lambda x: int(x.location.start) + 1, reverse=False)
            else:
                print_sub_feature = sorted([i for i in mRNA.sub_features if i.type != 'exon'],
                                           key=lambda x: int(x.location.start) + 1, reverse=True)

            for index in range(0, len(print_sub_feature)):
                feature_tmp = print_sub_feature[index]

                file.write("%s\t(ID: %s): \n" %
                           (feature_tmp.type, feature_tmp.id))
                start = int(feature_tmp.location.start) + 1
                end = int(feature_tmp.location.end)
                seq_tmp = contig_record.sub(start, end, strand, RNA=False)
                file.write(">%s:%d-%d:%s\n" % (rec.id, start, end, strand))
                file.write("%s\n\n" % seq_tmp)

                if index != len(print_sub_feature) - 1:

                    if strand == "+":
                        start = int(feature_tmp.location.end) + 1
                        feature_tmp_next = print_sub_feature[index + 1]
                        end = int(feature_tmp_next.location.start) + 1 - 1
                    else:
                        # start = int(feature_tmp.location.start) + 1 - 1
                        # feature_tmp_next = print_sub_feature[index + 1]
                        # end = int(feature_tmp_next.location.end) + 1
                        end = int(feature_tmp.location.start) + 1 - 1
                        feature_tmp_next = print_sub_feature[index + 1]
                        start = int(feature_tmp_next.location.end) + 1

                    if abs(start - end) > 1:
                        # print(start, end)
                        seq_tmp = contig_record.sub(
                            start, end, strand, RNA=False)
                        file.write("intron: \n")
                        file.write(">%s:%d-%d:%s\n" %
                                   (rec.id, start, end, strand))
                        file.write("%s\n\n" % seq_tmp)

            # downstream_range 3k
            file.write("downstream range (3k): \n")
            if strand == "+":
                start = int(mRNA.location.end) + 1
                end = int(mRNA.location.end) + 3001
            else:
                start = int(mRNA.location.start) + 1 - 3000
                end = int(mRNA.location.start) + 1 - 1
            seq_tmp = contig_record.sub(start, end, strand, RNA=False)
            file.write(">%s:%d-%d:%s\n" % (rec.id, start, end, strand))
            file.write("%s\n\n" % seq_tmp)

            num += 1

    else:
        file.write("# Can not find gene: %s\n" % gene.id)

# SeqMerge


def SeqMerge_main(args):
    """
    class abc():
        pass

    args = abc()

    args.ID_merge_file = "/lustre/home/xuyuxing/Database/Gel/genome/assembly/hic/adjust/merge.txt"
    args.db_fasta_file = "/lustre/home/xuyuxing/Database/Gel/genome/assembly/hic/adjust/Gel.genome.v1.0.FINAL.fasta"
    args.output_file = "/lustre/home/xuyuxing/Database/Gel/genome/assembly/hic/adjust/merged.fasta"
    """

    file_info = tsv_file_dict_parse(
        args.ID_merge_file, fieldnames=['old_id', 'new_id'])
    ID_dict = {}
    for i in file_info:
        old_id, new_id = file_info[i]['old_id'], file_info[i]['new_id']
        if new_id not in ID_dict:
            ID_dict[new_id] = []
        ID_dict[new_id].append(old_id)

    fasta_dict = read_fasta_by_faidx(args.db_fasta_file)

    merge_used_id = []
    for new_id in ID_dict:
        merge_seq = ""
        for old_id in ID_dict[new_id]:
            merge_seq += fasta_dict[old_id].seq
            merge_used_id.append(old_id)
        new_contig_seq = BioSeq(seq=merge_seq, seqname=new_id)
        ID_dict[new_id] = new_contig_seq
        # new_contig_seq.write_to_file(args.masked_ref_genome)

    for id in fasta_dict:
        if id not in merge_used_id:
            fasta_record = fasta_dict[id]
            fasta_record.write_to_file(args.output_file)

    for id in ID_dict:
        fasta_record = ID_dict[id]
        fasta_record.write_to_file(args.output_file)

# GCproportion


def GCproportion_main(args):
    """
    class abc():
        pass

    args = abc()

    args.fasta_file = "/lustre/home/xuyuxing/Database/Gel/genome/assembly/contig_filter/Gel.genome.v1.0.final.rename.fasta"
    args.output_file = "/lustre/home/xuyuxing/Database/Gel/genome/assembly/contig_filter/Gel.genome.v1.0.final.rename.GC.txt"
    """
    seq_dict = read_fasta_by_faidx(args.fasta_file)

    with open(args.output_file, 'w') as f:
        for contig_id in seq_dict:
            tmp_seq = str(seq_dict[contig_id].seq)
            GC_num = tmp_seq.count('C') + tmp_seq.count('G') + \
                tmp_seq.count('c') + tmp_seq.count('g')
            N_num = tmp_seq.count('N') + tmp_seq.count('n')
            GC_pro = GC_num / (len(tmp_seq) - N_num)
            f.write("%s\t%.5f\n" % (contig_id, GC_pro))

# SeqEntropy


def SeqEntropy_main(args):
    """
    class abc():
        pass

    args = abc()

    args.input_fasta = "//lustre/home/xuyuxing/Database/Plant_genome/clean_data/Phelipanche_aegyptiaca/no_clean/T99112N0.gene_model.protein.fasta"
    args.output_file = "/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Phelipanche_aegyptiaca/no_clean/T99112N0.gene_model.entropy.txt"
    args.word_size = 3
    """

    record_dict = read_fasta_by_faidx(args.input_fasta)

    with open(args.output_file, 'w') as f:
        for gene_id in record_dict:
            entropy = sb.sequence_entropy(
                record_dict[gene_id].seq, args.word_size)
            f.write("%s\t%f\n" % (gene_id, entropy))



import configparser
import logging
import math
import re
import time
from collections import OrderedDict
import json
import os
import pickle

from lib.common.os import *
from lib.common.util import *
from lib.xuyuxing.math.base import base_translate
from operator import attrgetter


def log_print(print_string):
    time_tmp = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime())
    print("%s\t\t\t%s" % (time_tmp, print_string))


def chunks(l, n):
    """Yield successive n-sized chunks from l."""
    for i in range(0, len(l), n):
        yield l[i:i + n]


def num_to_chr(num, digit):
    """
    when digit is 5:
    let 0 -> AAAAA
        1 -> AAAAB
    """

    base_list = base_translate(num, 26)

    output_list = []
    for i in [0]*(digit-len(base_list)) + base_list:
        output_list.append(chr(65+i))

    return ''.join(output_list)


def get_od(ordereddict_input, index_input):
    key_list = list(ordereddict_input.keys())
    index_output = key_list[index_input]
    return ordereddict_input[index_output]


# dict common tools
def dict2class(dict_input, class_target, attribute_list, key2attr=None):
    """
    This func can trans a dict with many object which attr recorded as a list in dict value
    to a dict with many object which recorded as a given class
    :param dict_input:      As dict output from tsv_file_parse, which key is a object id and value
                            is a list that many attr are recorded with a order but no name of
                            attribute.
    :param class_target:    which class you want
    :param attribute_list:  a list including name of attributes, order is same as dict_input
                            value and length of attribute_list should short or same as value
                            of dict_input.
    :param key2attr:        if key should record as an attr in class, give me the name.
    :return:    a dict with many object which recorded as a given class.
    """
    dict_output = {}

    for i in dict_input:
        key = i
        value_list = dict_input[i]
        dict_output[i] = class_target()

        if key2attr is not None:
            setattr(dict_output[i], key2attr, key)

        for rank in range(0, len(attribute_list)):
            if rank > len(value_list) - 1:
                break
            setattr(dict_output[i], attribute_list[rank], value_list[rank])

    return dict_output


def dict_slice(target_dict, slice_used, sort_flag=False, sort_key_function=None, reverse=False):
    """
    get something in dict as slice for a list, better for a OrderedDict
    :param target_dict: a dict, if it's a OrderedDict will be better
    :param slice_used: a object from slice function output. See https://docs.python.org/3/library/functions.html#slice
    :param sort_flag: do you want to sort keys in a dict
    :param sort_key_function: key args from sorted function
    :param reverse: if reverse
    :return:
    """

    if sort_flag:
        target_dict_sorted_keys = list(
            sorted(target_dict, key=sort_key_function, reverse=reverse))
    else:
        target_dict_sorted_keys = list(target_dict.keys())

    output_dir = OrderedDict()
    for i in target_dict_sorted_keys[slice_used]:
        output_dir[i] = target_dict[i]

    return output_dir


def merge_dict(dict_list, extend_value=True):
    """
    merge some dict into one dict

    extend_value:
    if same key find in diff dict, extend value as list, if extend_value is True, all value will be list

    else, old value will be delete

    a = {1:1,2:2,3:3}
    b = {1:2,4:4,5:5}

    merge_dict([a,b])
    Out[11]: OrderedDict([(1, [1, 2]), (2, [2]), (3, [3]), (4, [4]), (5, [5])])

    merge_dict([a,b], False)
    Out[12]: OrderedDict([(1, 2), (2, 2), (3, 3), (4, 4), (5, 5)])

    """

    output_dict = OrderedDict()

    for dict_tmp in dict_list:
        for i in dict_tmp:
            if extend_value:
                if not i in output_dict:
                    output_dict[i] = []

                if isinstance(dict_tmp[i], list):
                    output_dict[i].extend(dict_tmp[i])
                else:
                    output_dict[i].append(dict_tmp[i])
            else:
                output_dict[i] = dict_tmp[i]

    return output_dict


def hash_map(dict_list):
    dict_hash = {}
    for i in dict_list:
        for j in dict_list[i]:
            if j not in dict_hash:
                dict_hash[j] = []
            dict_hash[j].append(i)
    return dict_hash


def dict_key_value_interchange(dict_input):
    """interchange key and value, value should in list and uniq"""

    dict_output = {}
    for i in dict_input:
        for j in dict_input[i]:
            dict_output[j] = i

    return dict_output


"""
Abbreviation for the month
"""
month_abbr = ["Jan", "Feb", "Mar", "Apr", "May",
              "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"]


def flag_maker(bit_list):
    """
    input a bit list, like: [1,2,3,4,5] or [1,3,5], return a int as flag
    input int will be 1-base
    """
    flag = 0
    for i in bit_list:
        flag = flag + 2 ** (i - 1)
    return flag


def flag_parse(n):
    '''''convert denary integer n to binary string bStr, https://en.wikipedia.org/wiki/SAM_(file_format)#Bitwise_Flags'''
    bStr = ''
    if n < 0:
        raise ValueError("must be a positive integer")
    if n == 0:
        return []
    num = 0
    output = []
    while n > 0:
        num = num + 1
        if n % 2 == 1:
            output.append(num)
        n = n >> 1
    return output


def flag_filter(given_flag, need_flag, exclude_flag):
    """
    give a bitwise flags, and a flags which must have and a flag which should exclude
    :param given_flag: a int for bitwise flag
    :param need_flag: a int for need, nothing use 0
    :param exclude_flag: a int for exclude, nothing use 0
    :return: True or False
    """

    given_flag_list = set(flag_parse(given_flag))
    need_flag_list = set(flag_parse(need_flag))
    exclude_flag_list = set(flag_parse(exclude_flag))

    return (given_flag_list & need_flag_list == need_flag_list) and (given_flag_list & exclude_flag_list == set())


def millify(n):
    """
    let size in byte is easy for human, 1000 base
    :param n: 100000
    :return:  100 KB
    """
    millnames = ['', ' K', ' M', ' G', ' P']

    n = float(n)
    millidx = max(0, min(len(millnames) - 1,
                         int(math.floor(0 if n == 0 else math.log10(abs(n)) / 3))))

    return '{:.2f}{}'.format(n / 10 ** (3 * millidx), millnames[millidx])


# used to save pickle

# copy from flye
class Job(object):
    """
    Describes an abstract list of jobs with persistent
    status that can be resumed
    """
    run_params = {"stage_name": ""}

    def __init__(self):
        self.name = None
        self.args = None
        self.work_dir = None
        self.out_files = {}
        self.log_file = None

    def run(self):
        self.log_file.info(">>>STAGE: %s", self.name)

    def save(self, save_file):
        Job.run_params["stage_name"] = self.name

        with open(save_file, "w") as f:
            json.dump(Job.run_params, f)

    def load(self, save_file):
        with open(save_file, "r") as f:
            data = json.load(f)

            Job.run_params = data

    def completed(self, save_file):
        with open(save_file, "r") as f:
            dummy_data = json.load(f)

            for file_path in self.out_files.values():
                if not os.path.exists(file_path):
                    return False

            return True


if __name__ == '__main__':
    import argparse

    # argument parse
    parser = argparse.ArgumentParser(
        prog='TaxonTools',
    )

    subparsers = parser.add_subparsers(
        title='subcommands', dest="subcommand_name")

    # argparse for ID2Lineage
    parser_a = subparsers.add_parser('ID2Lineage',
                                     help='get taxonomy lineage by taxon ID', description='')
    parser_a.add_argument(
        "-o", "--output_dir", help="Output file name (default as $STDOUT)", default=None, type=str)
    parser_a.add_argument(
        "-c", "--contig_file", help="Output file name (default as $STDOUT)", default=None, type=str)

    args = parser.parse_args()
    args_dict = vars(args)

    # --------------------------------------------
    # command detail

    if args_dict["subcommand_name"] == "ID2Lineage":
        args = configure_parser(args,
                                "/lustre/home/xuyuxing/Database/Balanophora/Comp_genome/dongming/orthofinder_191218/output/config", args.contig_file, parameter_type_dict=None, parameter_parser_block=None)

        attrs = vars(args)

        for item in attrs.items():
            print("%s: %s" % item)



from collections import OrderedDict
import pickle
from lib.common.fileIO import tsv_file_dict_parse
from api.xuyuxing.file_parser.fileIO import tsv_file_parse


ortho_table = '/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S44/Orthogroups.tsv'
ol_dir = '/lustre/home/xuyuxing/Work/Gel/WPGmapper/Gel_ortho/protein/OrthoFinder/Results_Nov17/Orthologues'

gene_loss_table_dict = OrderedDict([
    ('Ash', ['T1088818N0.gene_model.protein', '/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S44/Ash/4.table_report.txt']),
    ('Dca', ['T906689N0.gene_model.protein', '/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S44/Dca/4.table_report.txt']),
    ('Peq', ['T78828N0.gene_model.protein', '/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S44/Peq/4.table_report.txt']),
    ('Gel', ['Gel.gene_model.protein', '/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S44/Gel/4.table_report.txt']),
    ('Cau', ['T267555N0.gene_model.protein', '/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S44/Cau/4.table_report.txt'])]
)


reference_genome = {
    'Ath' : ["T3702N0.gene_model.protein", "/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Arabidopsis_thaliana/function/all_annotation.pyb"],
    'Osa': ["T4530N0.gene_model.protein", "/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Oryza_sativa/function/all_annotation.pyb"]
}

orthogroup_dict = tsv_file_dict_parse(ortho_table,key_col='Orthogroup')

for spec in gene_loss_table_dict:
    table_report_file = gene_loss_table_dict[spec][1]
    tmp_info = tsv_file_parse(table_report_file, key_col=1, fields="1,2,3,4")
    gene_loss_table_dict[spec].append(tmp_info)

# Ath

TEMP = open(reference_genome['Ath'][1], 'rb')
Ath_annotation_dict = pickle.load(TEMP)
TEMP.close()

ath_gene_hash = {}
for OG_id in orthogroup_dict:
    ath_gene_id_list = orthogroup_dict[OG_id][reference_genome['Ath'][0]].split(", ")
    for g_id in ath_gene_id_list:
        ath_gene_hash[g_id] = [OG_id]

for g_id in ath_gene_hash:
    OG_id = ath_gene_hash[g_id][0]
    loss_flag_list = []
    for spec in gene_loss_table_dict:
        gene_loss_dict = gene_loss_table_dict[spec][2]

        if OG_id in gene_loss_dict:
            Conserved='yes'
            if gene_loss_dict[OG_id][2] == 'True':
                loss_flag_list.append('yes')
            else:
                loss_flag_list.append('no')
        else:
            loss_flag_list.append('no')
            Conserved='no'
    ath_gene_hash[g_id].append(Conserved)
    ath_gene_hash[g_id].extend(loss_flag_list)

# ol
ath_gene_ol_dict = {}
for spec in gene_loss_table_dict:
    ol_file = ol_dir + "/Orthologues_" + reference_genome['Ath'][0] + "/" + reference_genome['Ath'][0] + "__v__" + gene_loss_table_dict[spec][0] + ".tsv"
    ath_gene_ol_hash = {}
    tmp_info = tsv_file_parse(ol_file)
    for i in tmp_info:
        ath_gene_list = tmp_info[i][1].split(", ")
        for j in ath_gene_list:
            ath_gene_ol_hash[j] = tmp_info[i][2].split(",")
    ath_gene_ol_dict[spec] = ath_gene_ol_hash

    print(spec,max([len(ath_gene_ol_hash[i]) for i in ath_gene_ol_hash]))


ath_gene_og_dict = {}
for spec in gene_loss_table_dict:
    ath_gene_og_dict[spec] = {}
    for OG_id in orthogroup_dict:
        ath_gene_id_list = orthogroup_dict[OG_id][reference_genome['Ath'][0]].split(", ")
        spec_gene_id_list = orthogroup_dict[OG_id][gene_loss_table_dict[spec][0]].split(", ")
        for i in ath_gene_id_list:
            ath_gene_og_dict[spec][i] = spec_gene_id_list
    print(spec,max([len(ath_gene_og_dict[spec][i]) for i in ath_gene_og_dict[spec]]))


# output
from lib.common.util import printer_list

with open("/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S44/Ath.ref.gene_loss.tsv", 'w') as f:
    head_list = ["ref_id", "OrthoGroup_ID","Conserved", "Lossed_in_Ash","Lossed_in_Dca","Lossed_in_Peq","Lossed_in_Gel","Lossed_in_Cau","Gene_ID_in_Ash","Gene_ID_in_Dca","Gene_ID_in_Peq","Gene_ID_in_Gel","Gene_ID_in_Cau","Gene Name", "Alias", "Full Name", "key_words", "Computational Description", "PubMed_number",
                     "KEGG pathway(id)", "KEGG pathway", "KEGG module(id)", "KEGG module",  "KEGG KO(id)", "KEGG KO", "GO(id)", "GO_parent", "GO", "Plantcyc(id)", "Plantcyc",  "clean_id"]

    f.write(printer_list(head_list)+"\n")

    for g_id in Ath_annotation_dict:
        anno_list = Ath_annotation_dict[g_id]
        insert_info = []
        if g_id in ath_gene_hash:
            insert_info = ath_gene_hash[g_id]
        else:
            insert_info = ["-"] * 7

        for spec in gene_loss_table_dict:

            if g_id in ath_gene_ol_dict[spec]:
                ol_gene_list = ath_gene_ol_dict[spec][g_id]
            elif g_id in ath_gene_og_dict[spec]:
                ol_gene_list = ath_gene_og_dict[spec][g_id]
            else:
                ol_gene_list = []

            ol_gene_string = printer_list(ol_gene_list,';')

            insert_info.append(ol_gene_string)


        output_list = [anno_list[0]] + insert_info + list(anno_list[1:])

        f.write(printer_list(output_list)+"\n")



# Osa

TEMP = open(reference_genome['Osa'][1], 'rb')
Ath_annotation_dict = pickle.load(TEMP)
TEMP.close()

ath_gene_hash = {}
for OG_id in orthogroup_dict:
    ath_gene_id_list = orthogroup_dict[OG_id][reference_genome['Osa'][0]].split(", ")
    for g_id in ath_gene_id_list:
        ath_gene_hash[g_id] = [OG_id]

for g_id in ath_gene_hash:
    OG_id = ath_gene_hash[g_id][0]
    loss_flag_list = []
    for spec in gene_loss_table_dict:
        gene_loss_dict = gene_loss_table_dict[spec][2]

        if OG_id in gene_loss_dict:
            Conserved='yes'
            if gene_loss_dict[OG_id][2] == 'True':
                loss_flag_list.append('yes')
            else:
                loss_flag_list.append('no')
        else:
            loss_flag_list.append('no')
            Conserved='no'
    ath_gene_hash[g_id].append(Conserved)
    ath_gene_hash[g_id].extend(loss_flag_list)

# ol
ath_gene_ol_dict = {}
for spec in gene_loss_table_dict:
    ol_file = ol_dir + "/Orthologues_" + reference_genome['Osa'][0] + "/" + reference_genome['Osa'][0] + "__v__" + gene_loss_table_dict[spec][0] + ".tsv"
    ath_gene_ol_hash = {}
    tmp_info = tsv_file_parse(ol_file)
    for i in tmp_info:
        ath_gene_list = tmp_info[i][1].split(", ")
        for j in ath_gene_list:
            ath_gene_ol_hash[j] = tmp_info[i][2].split(",")
    ath_gene_ol_dict[spec] = ath_gene_ol_hash

    print(spec,max([len(ath_gene_ol_hash[i]) for i in ath_gene_ol_hash]))


ath_gene_og_dict = {}
for spec in gene_loss_table_dict:
    ath_gene_og_dict[spec] = {}
    for OG_id in orthogroup_dict:
        ath_gene_id_list = orthogroup_dict[OG_id][reference_genome['Ath'][0]].split(", ")
        spec_gene_id_list = orthogroup_dict[OG_id][gene_loss_table_dict[spec][0]].split(", ")
        for i in ath_gene_id_list:
            ath_gene_og_dict[spec][i] = spec_gene_id_list
    print(spec,max([len(ath_gene_og_dict[spec][i]) for i in ath_gene_og_dict[spec]]))


# output
from lib.common.util import printer_list

with open("/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S44/Osa.ref.gene_loss.tsv", 'w') as f:
    head_list = ["ref_id", "OrthoGroup_ID","Conserved", "Lossed_in_Ash","Lossed_in_Dca","Lossed_in_Peq","Lossed_in_Gel","Lossed_in_Cau","Gene_ID_in_Ash","Gene_ID_in_Dca","Gene_ID_in_Peq","Gene_ID_in_Gel","Gene_ID_in_Cau"] + ["Gene Name", "Alias", "Full Name", "key_words", "Computational Description", "PubMed_number",
                     "KEGG pathway(id)", "KEGG pathway", "KEGG module(id)", "KEGG module",  "KEGG KO(id)", "KEGG KO", "GO(id)", "GO_parent", "GO", "Plantcyc(id)", "Plantcyc",  "clean_id"]


    f.write(printer_list(head_list)+"\n")

    for g_id in Ath_annotation_dict:
        anno_list = Ath_annotation_dict[g_id]
        insert_info = []
        if g_id in ath_gene_hash:
            insert_info = ath_gene_hash[g_id]
        else:
            insert_info = ["-"] * 7

        for spec in gene_loss_table_dict:

            if g_id in ath_gene_ol_dict[spec]:
                ol_gene_list = ath_gene_ol_dict[spec][g_id]
            elif g_id in ath_gene_og_dict[spec]:
                ol_gene_list = ath_gene_og_dict[spec][g_id]
            else:
                ol_gene_list = []

            ol_gene_string = printer_list(ol_gene_list,';')

            insert_info.append(ol_gene_string)


        output_list = [anno_list[0]] + insert_info + list(anno_list[1:])

        f.write(printer_list(output_list)+"\n")





# coding:utf-8
import matplotlib

matplotlib.use('Agg')
import matplotlib.pyplot as plt
import sys
sys.path.append("/lustre/home/xuyuxing/python_project/Genome_work_tools/")
from lib.common.fileIO import tsv_file_dict_parse
data_file = "/lustre/home/xuyuxing/Work/Other/tmp/printdata.txt"

data_dict = tsv_file_dict_parse(data_file)

x=sys.argv[1]
y=sys.argv[2]

S02_S01_unknown_peak = [float(data_dict[i][x]) for i in data_dict if (data_dict[i]["Known"] == "" and data_dict[i]["Detected"] == "")]
S02_S01_known_peak = [float(data_dict[i][x]) for i in data_dict if data_dict[i]["Known"] == "6"]
S02_S01_Detected_peak = [float(data_dict[i][x]) for i in data_dict if data_dict[i]["Detected"] == "6"]

S06_S05_unknown_peak = [float(data_dict[i][y]) for i in data_dict if (data_dict[i]["Known"] == "" and data_dict[i]["Detected"] == "")]
S06_S05_known_peak = [float(data_dict[i][y]) for i in data_dict if data_dict[i]["Known"] == "6"]
S06_S05_Detected_peak = [float(data_dict[i][y]) for i in data_dict if data_dict[i]["Detected"] == "6"]

plt.plot(S02_S01_unknown_peak, S06_S05_unknown_peak, 'bo', markersize=3, label='Other sites')
plt.plot(S02_S01_Detected_peak, S06_S05_Detected_peak, 'yo', markersize=3, label='Detected')
plt.plot(S02_S01_known_peak, S06_S05_known_peak, 'ro', markersize=3, label='Published')

plt.legend(loc='upper left')

plt.xlabel(x)
plt.ylabel(y)

plt.savefig('/lustre/home/xuyuxing/Work/Other/tmp/%s.vs.%s.pdf' % (x,y), dpi=1000)



from Bio import GenBank
from lib.common.genome.genome_feature2 import GenomeFeature, ChrLoci, write_gff_file
import re


def parse_location_string(location_string):
    if re.match(r'^complement.*', location_string):
        mobj = re.match(r'^complement\((.*)\)', location_string)
        range_string = mobj.groups()[0]
        range_list = parse_location_string(range_string)
        range_list = [(i[0], '-') for i in range_list]

    elif re.match(r'^join.*', location_string):
        mobj = re.match(r'^join\((.*)\)', location_string)
        range_list = []
        for sub_string in mobj.groups()[0].split(","):
            sub_range_list = parse_location_string(sub_string)
            range_list.extend(sub_range_list)

    elif re.match(r'(\d+)\.\.(\d+)', location_string):
        strand = '+'
        mobj = re.match(r'(\d+)\.\.(\d+)', location_string)
        range_now = tuple(
            [tuple(sorted([int(i) for i in mobj.groups()])), strand])
        range_list = [range_now]

    return range_list


def GeneBank2Gff3_main(args):
    gene_name_prefix = 'GNPREF'
    gene_feautre_dict = {}

    with open(args.GeneBank_file) as handle:
        for record in GenBank.parse(handle):
            record_id = record.accession[0]
            record_seq = record.sequence

            gene = None

            num = 0
            for feature in record.features:
                ft_qual_dict = {}

                for i in feature.qualifiers:
                    key_now = i.key.replace('/', '').replace('=', '')
                    ft_qual_dict[key_now] = i.value

                    if 'translation' in ft_qual_dict:
                        del ft_qual_dict['translation']
                    if 'annotator' in ft_qual_dict:
                        del ft_qual_dict['annotator']

                if feature.key == 'gene':
                    if not gene is None:
                        gene_feautre_dict[g_id] = gene

                    g_id = "%s_%s_%05d" % (gene_name_prefix, record_id, num)

                    location = feature.location
                    loc_list = parse_location_string(location)
                    gf_range = (min([i[0][0] for i in loc_list]),
                                max([i[0][1] for i in loc_list]))
                    strand = loc_list[0][1]

                    gene = GenomeFeature(id=g_id, type='gene', chr_loci=ChrLoci(chr_id=record_id, strand=strand, start=gf_range[0], end=gf_range[1]),
                                         qualifiers=ft_qual_dict, sub_features=[])

                    num += 1

                elif feature.key in ['CDS', 'tRNA', 'rRNA']:
                    m_id = gene.id + "." + str(len(gene.sub_features))
                    location = feature.location
                    loc_list = parse_location_string(location)
                    gf_range = (min([i[0][0] for i in loc_list]),
                                max([i[0][1] for i in loc_list]))
                    strand = loc_list[0][1]

                    if feature.key == 'CDS':
                        gf = GenomeFeature(id=m_id, type='mRNA', chr_loci=ChrLoci(chr_id=record_id, strand=strand, start=gf_range[0], end=gf_range[1]),
                                           qualifiers=ft_qual_dict, sub_features=[])
                    else:
                        gf = GenomeFeature(id=m_id, type=feature.key, chr_loci=ChrLoci(chr_id=record_id, strand=strand, start=gf_range[0], end=gf_range[1]),
                                           qualifiers=ft_qual_dict, sub_features=[])

                    exon_num = 0
                    for exon_range, strand in loc_list:
                        e_id = gf.id + ".exon." + str(exon_num)
                        exon = GenomeFeature(id=e_id, type='exon', chr_loci=ChrLoci(
                            chr_id=record_id, strand=strand, start=exon_range[0], end=exon_range[1]))
                        gf.sub_features.append(exon)

                        if feature.key == 'CDS':
                            c_id = gf.id + ".cds." + str(exon_num)
                            cds = GenomeFeature(id=c_id, type='CDS', chr_loci=ChrLoci(
                                chr_id=record_id, strand=strand, start=exon_range[0], end=exon_range[1]))
                            gf.sub_features.append(cds)

                        exon_num += 1

                    gene.sub_features.append(gf)

    write_gff_file([gene_feautre_dict[i] for i in gene_feautre_dict],
                   args.output_gff3, source="GeneBank", sort=True)


if __name__ == '__main__':
    class abc():
        pass

    args = abc()
    args.GeneBank_file = '/lustre/home/xuyuxing/Database/Orobanche/hic/filter/Ocu_organelle_genome/Chl_annotation/GeSeqJob-20210802-95230_tig00121106_debris_2581-92462_GenBank.txt'
    args.output_gff3 = '/lustre/home/xuyuxing/Database/Orobanche/hic/filter/Ocu_organelle_genome/Chl_annotation/ChrC'



from api.xuyuxing.resource.kegg.kegg_config import *
import urllib.request
import os
import errno
import sys
from lib.common.os import mkdir
from lib.common.fileIO import tsv_file_dict_parse, read_list_file, tsv_file_dict_parse_big
from lib.common.util import printer_list
from lib.common.math.bin_and_window import split_sequence_to_bins

# Do not change!

TAX_URL = 'http://rest.kegg.jp/list/organism'
MODULE_URL = 'http://rest.kegg.jp/list/module'
PATHWAY_URL = 'http://rest.kegg.jp/list/pathway'
KO_URL = "http://rest.kegg.jp/list/ko"

GENOME_LINK_URL = "http://rest.kegg.jp/link/genome/"
PATHWAY_LINK_URL = 'http://rest.kegg.jp/link/pathway/'
MODULE_LINK_URL = 'http://rest.kegg.jp/link/module/'
KO_LINK_URL = "http://rest.kegg.jp/link/ko/"


def get_url_into_file(url_input, output_file_name):
    try:
        os.remove(output_file_name)
    except:
        pass

    urllib.request.urlretrieve(url_input, output_file_name)

    return output_file_name


def get_url_into_string(url_input):
    tmp_file, tmp_out = urllib.request.urlretrieve(url_input)

    with open(tmp_file, 'r') as f:
        string_output = f.read()

    os.remove(tmp_file)

    return string_output


def get_module_list(output_file_name):
    """
    Get module detail info file with all organism codes from KEGG.
    http://rest.kegg.jp/list/module
    """
    get_url_into_file(MODULE_URL, output_file_name + ".tmp")

    try:
        os.remove(output_file_name)
    except:
        pass

    with open(output_file_name + '.tmp', 'r') as fin, open(
        output_file_name, 'w') as fout:
        for line in fin.readlines():
            module = line.split('\t')[0].split(':')[1]
            name = line.split('\t')[1]
            fout.write(module + '\t' + name)

    os.remove(output_file_name + '.tmp')

    return output_file_name


def get_pathway_list(output_file_name):
    """
    Get pathway detail info file with all organism codes from KEGG.
    http://rest.kegg.jp/list/pathway
    """
    get_url_into_file(PATHWAY_URL, output_file_name + ".tmp")

    try:
        os.remove(output_file_name)
    except:
        pass

    with open(output_file_name + '.tmp', 'r') as fin, open(
        output_file_name, 'w') as fout:
        for line in fin.readlines():
            module = line.split('\t')[0].split(':')[1]
            name = line.split('\t')[1]
            fout.write(module + '\t' + name)

    os.remove(output_file_name + '.tmp')

    return output_file_name


def get_taxonomy_code(output_file_name):
    """
    Get taxonomy file with all organism codes from KEGG.
    http://rest.kegg.jp/list/organism
    """
    return get_url_into_file(TAX_URL, output_file_name)
    


def parse_taxon_file(taxon_file):
    taxon_dict = tsv_file_dict_parse(taxon_file,key_col='id',fieldnames=['id','code','species','taxonomy'])
    return taxon_dict


def get_species_gene_list(species_code, output_file_name):
    """
    Get gene list of one species from KEGG.
    http://rest.kegg.jp/link/genome/ath
    """
    get_url_into_file(GENOME_LINK_URL + species_code, output_file_name + ".tmp")

    try:
        os.remove(output_file_name)
    except:
        pass

    with open(output_file_name + '.tmp', 'r') as fin, open(
        output_file_name, 'w') as fout:
        for line in fin.readlines():
            gene_name = line.split('\t')[0]
            fout.write(gene_name + '\n')

    os.remove(output_file_name + '.tmp')

    return output_file_name


def parse_gene_list(gene_list_file):
    return read_list_file(gene_list_file)


def get_species_gene_pathway(species_code,output_file_name):

    get_url_into_file(PATHWAY_LINK_URL + species_code, output_file_name + ".pathway_link")
    get_url_into_file(PATHWAY_URL + "/" + species_code, output_file_name + ".pathway_list")
    
    pathway_info = tsv_file_dict_parse(output_file_name + ".pathway_list", fieldnames=['p_id','info'], key_col='p_id')
    pathway_link = tsv_file_dict_parse(output_file_name + ".pathway_link", fieldnames=['id','p_id'], key_col='id')

    with open(output_file_name, 'w') as f:
        for i in pathway_link:
            g_id  = i
            p_id = pathway_link[i]['p_id']
            p_info = pathway_info[p_id]['info']
            f.write("%s\t%s\t%s\n" % (g_id, p_id, p_info))
    
    os.remove(output_file_name + ".pathway_list")
    os.remove(output_file_name + ".pathway_link")

    return output_file_name


def get_species_gene_module(species_code,output_file_name):

    get_url_into_file(MODULE_LINK_URL + species_code, output_file_name + ".module_link")
    get_url_into_file(MODULE_URL, output_file_name + ".module_list")
    
    module_info = tsv_file_dict_parse(output_file_name + ".module_list", fieldnames=['m_id','info'], key_col='m_id')
    module_link = tsv_file_dict_parse(output_file_name + ".module_link", fieldnames=['id','m_id'], key_col='id')

    with open(output_file_name, 'w') as f:
        for i in module_link:
            g_id  = i
            m_id = module_link[i]['m_id']
            m_id = "md:" + m_id.split(":")[1].split("_")[1]
            m_info = module_info[m_id]['info']
            f.write("%s\t%s\t%s\n" % (g_id, m_id, m_info))
    
    os.remove(output_file_name + ".module_list")
    os.remove(output_file_name + ".module_link")

    return output_file_name



def get_species_gene_ko(species_code,output_file_name):

    get_url_into_file(KO_LINK_URL + species_code, output_file_name + ".ko_link")
    get_url_into_file(KO_URL, output_file_name + ".ko_list")
    
    ko_info = tsv_file_dict_parse(output_file_name + ".ko_list", fieldnames=['k_id','info'], key_col='k_id')
    ko_link = tsv_file_dict_parse(output_file_name + ".ko_link", fieldnames=['id','k_id'], key_col='id')

    with open(output_file_name, 'w') as f:
        for i in ko_link:
            g_id  = i
            k_id = ko_link[i]['k_id']
            if k_id in ko_info:
                k_info = ko_info[k_id]['info']
                f.write("%s\t%s\t%s\n" % (g_id, k_id, k_info))
            else:
                f.write("%s\t%s\t%s\n" % (g_id, k_id, 'None'))
            
    
    os.remove(output_file_name + ".ko_list")
    os.remove(output_file_name + ".ko_link")

    return output_file_name



def aa_seq_url(gene_list):
    gene_string = printer_list(gene_list,sep='+')
    url_string = "http://rest.kegg.jp/get/"+gene_string+"/aaseq"
    return url_string

def covert_url(gene_list):
    gene_string = printer_list(gene_list,sep='+')
    url_string = "http://rest.kegg.jp/conv/ncbi-geneid/"+gene_string
    return url_string

def covert_url_p(gene_list):
    gene_string = printer_list(gene_list,sep='+')
    url_string = "http://rest.kegg.jp/conv/ncbi-proteinid/"+gene_string
    return url_string

def ncbi_gene_id_vs_protein_accession_map(gene2accession_file, silence=True):
    protein_id_vs_gene_id_map = {}
    
    num = 0
    for id_tmp, info_dict in tsv_file_dict_parse_big(gene2accession_file, gzip_flag=True):
        
        p_id = info_dict['protein_accession.version']
        gi = info_dict['GeneID']
        protein_id_vs_gene_id_map[p_id] = gi

        num += 1
        if num % 100000 == 0 and not silence:
            print(num)

    return protein_id_vs_gene_id_map


def gene_id_convert(gene_list,function,step=100,silence=True):
    gene_id_map = {}

    for i in range(0,len(gene_list),step):
        if not silence:
            print(i)
        sub_gene_list = gene_list[i:i+step]
        url_string = function(sub_gene_list)
        try:
            output_string = get_url_into_string(url_string)
            for line in output_string.split("\n"):
                raw_id, new_id = line.split("\t")
                new_id = new_id.split(":")[1]

                gene_id_map[raw_id] = new_id

        except:
            pass

    return gene_id_map


def get_species_gene_aa(species_code,output_file_name,step=100):

    get_species_gene_list(species_code, output_file_name+".id")
    gene_list = parse_gene_list(output_file_name+".id")

    try:
        os.remove(output_file_name)
    except:
        pass

    for i in range(0,len(gene_list),step):
        sub_gene_list = gene_list[i:i+step]
        url_string = aa_seq_url(sub_gene_list)
        try:
            get_url_into_file(url_string, output_file_name+".tmp.aa")

            with open(output_file_name + '.tmp.aa', 'r') as fin, open(
                output_file_name, 'a') as fout:
                for line in fin.readlines():
                    fout.write(line)

            os.remove(output_file_name+".tmp.aa")

        except:
            pass

    os.remove(output_file_name+".id")

    return output_file_name
    

# Main
if __name__ == '__main__':

    from lib.xuyuxing.base.base_function import mkdir
    from lib.common.genome.seq_base import read_fasta_by_faidx

    species_code = 'ath'
    work_dir = '/lustre/home/xuyuxing/Database/kegg/ath'
    ncbi_aa_fasta = '/lustre/home/xuyuxing/Database/kegg/GCF_000001735.4_TAIR10.1_protein.faa'
    gene2accession_file = '/lustre/home/xuyuxing/Database/NCBI/gene/2020/gene2accession.gz'
    
    mkdir(work_dir, True)
    kegg_gene_id_file = work_dir + "/" + species_code + ".kegg_gene_id.txt"
    get_species_gene_list(species_code, kegg_gene_id_file)
    gene_list = parse_gene_list(kegg_gene_id_file)

    # get pathway
    kegg_pathway_file = work_dir + "/" + species_code + ".kegg_pathway.txt"
    get_species_gene_pathway(species_code, kegg_pathway_file)
    pathway_dict = tsv_file_dict_parse(kegg_pathway_file,fieldnames=['gene_id','id','des'],key_col='gene_id')

    # get module
    kegg_module_file = work_dir + "/" + species_code + ".kegg_module.txt"
    get_species_gene_module(species_code, kegg_module_file)
    module_dict = tsv_file_dict_parse(kegg_module_file,fieldnames=['gene_id','id','des'],key_col='gene_id')

    # get ko
    kegg_ko_file = work_dir + "/" + species_code + ".kegg_ko.txt"
    get_species_gene_ko(species_code, kegg_ko_file)
    ko_dict = tsv_file_dict_parse(kegg_ko_file,fieldnames=['gene_id','id','des'],key_col='gene_id')

    # get seq
    ## get ncbi gi
    ncbi_gi_map = gene_id_convert(gene_list,covert_url,step=100,silence=False)
    from lib.common.genome.seq_base import read_fasta_by_faidx
    fasta_record_dict = read_fasta_by_faidx(ncbi_aa_fasta)
    protein_id_vs_gene_id_map = ncbi_gene_id_vs_protein_accession_map(gene2accession_file, silence=True)
    
    gi_seq_dict = {}
    for ncbi_p_id in fasta_record_dict:
        if ncbi_p_id in protein_id_vs_gene_id_map:
            gi = protein_id_vs_gene_id_map[ncbi_p_id]
            if not gi in gi_seq_dict:
                gi_seq_dict[gi] = fasta_record_dict[ncbi_p_id]
            else:
                if fasta_record_dict[ncbi_p_id].len() > gi_seq_dict[gi].len():
                    gi_seq_dict[gi] = fasta_record_dict[ncbi_p_id]


    # merge info
    kegg_gene_dict = {}
    for kg_id in ncbi_gi_map:
        if kg_id in pathway_dict:
            pw_id = pathway_dict[kg_id]['id']
            pw_info = pathway_dict[kg_id]['des']
        else:
            pw_id = None
            pw_info = None

        if kg_id in module_dict:
            md_id = module_dict[kg_id]['id']
            md_info = module_dict[kg_id]['des']
        else:
            md_id = None
            md_info = None

        if kg_id in ko_dict:
            ko_id = ko_dict[kg_id]['id']
            ko_info = ko_dict[kg_id]['des']
        else:
            ko_id = None
            ko_info = None

        gi = ncbi_gi_map[kg_id]
        
        if gi in gi_seq_dict:
            seq = gi_seq_dict[gi]
            ncbi_id = gi_seq_dict[gi].seqname.split()[0]
        else:
            seq = None
            ncbi_id = None
        
        kegg_gene_dict[kg_id] = (kg_id,ncbi_id,pw_id,pw_info,md_id,md_info,ko_id,ko_info)
        
    kegg_overview_table = work_dir + "/" + species_code + ".overview.tsv"
    with open(kegg_overview_table, 'w') as f:
        for kg_id in kegg_gene_dict:
            f.write(printer_list(kegg_gene_dict[kg_id])+"\n")



    ## osa

    species_code = 'osa'
    work_dir = '/lustre/home/xuyuxing/Database/kegg/osa'
    ncbi_aa_fasta = '/lustre/home/xuyuxing/Database/kegg/GCF_000005425.2_Build_4.0_protein.faa'
    gene2accession_file = '/lustre/home/xuyuxing/Database/NCBI/gene/2020/gene2accession.gz'
    
    mkdir(work_dir, True)
    kegg_gene_id_file = work_dir + "/" + species_code + ".kegg_gene_id.txt"
    get_species_gene_list(species_code, kegg_gene_id_file)
    gene_list = parse_gene_list(kegg_gene_id_file)

    # get pathway
    kegg_pathway_file = work_dir + "/" + species_code + ".kegg_pathway.txt"
    get_species_gene_pathway(species_code, kegg_pathway_file)
    pathway_dict = tsv_file_dict_parse(kegg_pathway_file,fieldnames=['gene_id','id','des'],key_col='gene_id')

    # get module
    kegg_module_file = work_dir + "/" + species_code + ".kegg_module.txt"
    get_species_gene_module(species_code, kegg_module_file)
    module_dict = tsv_file_dict_parse(kegg_module_file,fieldnames=['gene_id','id','des'],key_col='gene_id')

    # get ko
    kegg_ko_file = work_dir + "/" + species_code + ".kegg_ko.txt"
    get_species_gene_ko(species_code, kegg_ko_file)
    ko_dict = tsv_file_dict_parse(kegg_ko_file,fieldnames=['gene_id','id','des'],key_col='gene_id')

    # get seq
    ## get ncbi gi
    ncbi_gi_map = gene_id_convert(gene_list,covert_url_p,step=100,silence=False)
    
    # merge info
    kegg_gene_dict = {}
    for kg_id in gene_list:
        if kg_id in pathway_dict:
            pw_id = pathway_dict[kg_id]['id']
            pw_info = pathway_dict[kg_id]['des']
        else:
            pw_id = None
            pw_info = None

        if kg_id in module_dict:
            md_id = module_dict[kg_id]['id']
            md_info = module_dict[kg_id]['des']
        else:
            md_id = None
            md_info = None

        if kg_id in ko_dict:
            ko_id = ko_dict[kg_id]['id']
            ko_info = ko_dict[kg_id]['des']
        else:
            ko_id = None
            ko_info = None

        if kg_id in ncbi_gi_map:
            ncbi_pid = ncbi_gi_map[kg_id]
        else:
            ncbi_pid = None
        
        kegg_gene_dict[kg_id] = (kg_id,ncbi_pid,pw_id,pw_info,md_id,md_info,ko_id,ko_info)
    
    kegg_overview_table = work_dir + "/" + species_code + ".overview.tsv"
    with open(kegg_overview_table, 'w') as f:
        for kg_id in kegg_gene_dict:
            f.write(printer_list(kegg_gene_dict[kg_id])+"\n")


    

